from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="NotificationsUserProfile")


@attr.s(auto_attribs=True)
class NotificationsUserProfile:
    """Represents the notification settings in the profile of the specific user."""

    notify_on_own_changes: "Union[Unset, bool]" = UNSET
    jabber_notifications_enabled: "Union[Unset, bool]" = UNSET
    email_notifications_enabled: "Union[Unset, bool]" = UNSET
    mention_notifications_enabled: "Union[Unset, bool]" = UNSET
    duplicate_cluster_notifications_enabled: "Union[Unset, bool]" = UNSET
    mailbox_integration_notifications_enabled: "Union[Unset, bool]" = UNSET
    use_plain_text_emails: "Union[Unset, bool]" = UNSET
    auto_watch_on_comment: "Union[Unset, bool]" = UNSET
    auto_watch_on_create: "Union[Unset, bool]" = UNSET
    auto_watch_on_vote: "Union[Unset, bool]" = UNSET
    auto_watch_on_update: "Union[Unset, bool]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        notify_on_own_changes = self.notify_on_own_changes
        jabber_notifications_enabled = self.jabber_notifications_enabled
        email_notifications_enabled = self.email_notifications_enabled
        mention_notifications_enabled = self.mention_notifications_enabled
        duplicate_cluster_notifications_enabled = self.duplicate_cluster_notifications_enabled
        mailbox_integration_notifications_enabled = self.mailbox_integration_notifications_enabled
        use_plain_text_emails = self.use_plain_text_emails
        auto_watch_on_comment = self.auto_watch_on_comment
        auto_watch_on_create = self.auto_watch_on_create
        auto_watch_on_vote = self.auto_watch_on_vote
        auto_watch_on_update = self.auto_watch_on_update
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if notify_on_own_changes is not UNSET:
            field_dict["notifyOnOwnChanges"] = notify_on_own_changes
        if jabber_notifications_enabled is not UNSET:
            field_dict["jabberNotificationsEnabled"] = jabber_notifications_enabled
        if email_notifications_enabled is not UNSET:
            field_dict["emailNotificationsEnabled"] = email_notifications_enabled
        if mention_notifications_enabled is not UNSET:
            field_dict["mentionNotificationsEnabled"] = mention_notifications_enabled
        if duplicate_cluster_notifications_enabled is not UNSET:
            field_dict["duplicateClusterNotificationsEnabled"] = duplicate_cluster_notifications_enabled
        if mailbox_integration_notifications_enabled is not UNSET:
            field_dict["mailboxIntegrationNotificationsEnabled"] = mailbox_integration_notifications_enabled
        if use_plain_text_emails is not UNSET:
            field_dict["usePlainTextEmails"] = use_plain_text_emails
        if auto_watch_on_comment is not UNSET:
            field_dict["autoWatchOnComment"] = auto_watch_on_comment
        if auto_watch_on_create is not UNSET:
            field_dict["autoWatchOnCreate"] = auto_watch_on_create
        if auto_watch_on_vote is not UNSET:
            field_dict["autoWatchOnVote"] = auto_watch_on_vote
        if auto_watch_on_update is not UNSET:
            field_dict["autoWatchOnUpdate"] = auto_watch_on_update
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        notify_on_own_changes = d.pop("notifyOnOwnChanges", UNSET)

        jabber_notifications_enabled = d.pop("jabberNotificationsEnabled", UNSET)

        email_notifications_enabled = d.pop("emailNotificationsEnabled", UNSET)

        mention_notifications_enabled = d.pop("mentionNotificationsEnabled", UNSET)

        duplicate_cluster_notifications_enabled = d.pop("duplicateClusterNotificationsEnabled", UNSET)

        mailbox_integration_notifications_enabled = d.pop("mailboxIntegrationNotificationsEnabled", UNSET)

        use_plain_text_emails = d.pop("usePlainTextEmails", UNSET)

        auto_watch_on_comment = d.pop("autoWatchOnComment", UNSET)

        auto_watch_on_create = d.pop("autoWatchOnCreate", UNSET)

        auto_watch_on_vote = d.pop("autoWatchOnVote", UNSET)

        auto_watch_on_update = d.pop("autoWatchOnUpdate", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        notifications_user_profile = cls(
            notify_on_own_changes=notify_on_own_changes,
            jabber_notifications_enabled=jabber_notifications_enabled,
            email_notifications_enabled=email_notifications_enabled,
            mention_notifications_enabled=mention_notifications_enabled,
            duplicate_cluster_notifications_enabled=duplicate_cluster_notifications_enabled,
            mailbox_integration_notifications_enabled=mailbox_integration_notifications_enabled,
            use_plain_text_emails=use_plain_text_emails,
            auto_watch_on_comment=auto_watch_on_comment,
            auto_watch_on_create=auto_watch_on_create,
            auto_watch_on_vote=auto_watch_on_vote,
            auto_watch_on_update=auto_watch_on_update,
            id=id,
            type=type,
        )

        return notifications_user_profile



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="LocalizableBundleElement")


try:
    from ..models import bundle_element
except ImportError:
    import sys

    bundle_element = sys.modules[__package__ + "bundle_element"]


@attr.s(auto_attribs=True)
class LocalizableBundleElement(bundle_element.BundleElement):
    """Represents field value that can be localized."""

    localized_name: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        localized_name = self.localized_name

        field_dict: Dict[str, Any] = {}
        _BundleElement_dict = super().to_dict()
        field_dict.update(_BundleElement_dict)
        field_dict.update({})
        if localized_name is not UNSET:
            field_dict["localizedName"] = localized_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        _BundleElement_kwargs = super().from_dict(src_dict=d).to_dict()
        _BundleElement_kwargs.pop("$type")

        localized_name = d.pop("localizedName", UNSET)

        localizable_bundle_element = cls(
            localized_name=localized_name,
            **_BundleElement_kwargs,
        )

        return localizable_bundle_element



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="VersionBundleCustomFieldDefaults")


try:
    from ..models import bundle_custom_field_defaults
except ImportError:
    import sys

    bundle_custom_field_defaults = sys.modules[__package__ + "bundle_custom_field_defaults"]


@attr.s(auto_attribs=True)
class VersionBundleCustomFieldDefaults(bundle_custom_field_defaults.BundleCustomFieldDefaults):
    """Default settings for the version-type field."""

    bundle: "Union[Unset, version_bundle_m.VersionBundle]" = UNSET
    default_values: "Union[Unset, List[version_bundle_element_m.VersionBundleElement]]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        bundle: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.bundle, Unset):
            bundle = self.bundle.to_dict()

        default_values: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.default_values, Unset):
            default_values = []
            for default_values_item_data in self.default_values:
                default_values_item = default_values_item_data.to_dict()

                default_values.append(default_values_item)

        field_dict: Dict[str, Any] = {}
        _BundleCustomFieldDefaults_dict = super().to_dict()
        field_dict.update(_BundleCustomFieldDefaults_dict)
        field_dict.update({})
        if bundle is not UNSET:
            field_dict["bundle"] = bundle
        if default_values is not UNSET:
            field_dict["defaultValues"] = default_values

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import version_bundle as version_bundle_m
            from ..models import version_bundle_element as version_bundle_element_m
        except ImportError:
            import sys

            version_bundle_m = sys.modules[__package__ + "version_bundle"]
            version_bundle_element_m = sys.modules[__package__ + "version_bundle_element"]

        d = src_dict.copy()

        _BundleCustomFieldDefaults_kwargs = super().from_dict(src_dict=d).to_dict()
        _BundleCustomFieldDefaults_kwargs.pop("$type")

        _bundle = d.pop("bundle", UNSET)
        bundle: Union[Unset, version_bundle_m.VersionBundle]
        if isinstance(_bundle, Unset):
            bundle = UNSET
        else:
            bundle = version_bundle_m.VersionBundle.from_dict(_bundle)

        default_values = []
        _default_values = d.pop("defaultValues", UNSET)
        for default_values_item_data in _default_values or []:
            default_values_item = version_bundle_element_m.VersionBundleElement.from_dict(default_values_item_data)

            default_values.append(default_values_item)

        version_bundle_custom_field_defaults = cls(
            bundle=bundle,
            default_values=default_values,
            **_BundleCustomFieldDefaults_kwargs,
        )

        return version_bundle_custom_field_defaults



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="IssueBasedSwimlaneSettings")


try:
    from ..models import swimlane_settings
except ImportError:
    import sys

    swimlane_settings = sys.modules[__package__ + "swimlane_settings"]


@attr.s(auto_attribs=True)
class IssueBasedSwimlaneSettings(swimlane_settings.SwimlaneSettings):
    """Swimlane settings for the case, when each swimlane is represented by an issue and contains this issue's subtasks."""

    field: "Union[Unset, filter_field_m.FilterField]" = UNSET
    default_card_type: "Union[Unset, swimlane_value_m.SwimlaneValue]" = UNSET
    values: "Union[Unset, List[swimlane_value_m.SwimlaneValue]]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        field: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.field, Unset):
            field = self.field.to_dict()

        default_card_type: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.default_card_type, Unset):
            default_card_type = self.default_card_type.to_dict()

        values: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.values, Unset):
            values = []
            for values_item_data in self.values:
                values_item = values_item_data.to_dict()

                values.append(values_item)

        field_dict: Dict[str, Any] = {}
        _SwimlaneSettings_dict = super().to_dict()
        field_dict.update(_SwimlaneSettings_dict)
        field_dict.update({})
        if field is not UNSET:
            field_dict["field"] = field
        if default_card_type is not UNSET:
            field_dict["defaultCardType"] = default_card_type
        if values is not UNSET:
            field_dict["values"] = values

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import filter_field as filter_field_m
            from ..models import swimlane_value as swimlane_value_m
        except ImportError:
            import sys

            swimlane_value_m = sys.modules[__package__ + "swimlane_value"]
            filter_field_m = sys.modules[__package__ + "filter_field"]

        d = src_dict.copy()

        _SwimlaneSettings_kwargs = super().from_dict(src_dict=d).to_dict()
        _SwimlaneSettings_kwargs.pop("$type")

        _field = d.pop("field", UNSET)
        field: Union[Unset, filter_field_m.FilterField]
        if isinstance(_field, Unset):
            field = UNSET
        else:
            field = filter_field_m.FilterField.from_dict(_field)

        _default_card_type = d.pop("defaultCardType", UNSET)
        default_card_type: Union[Unset, swimlane_value_m.SwimlaneValue]
        if isinstance(_default_card_type, Unset):
            default_card_type = UNSET
        else:
            default_card_type = swimlane_value_m.SwimlaneValue.from_dict(_default_card_type)

        values = []
        _values = d.pop("values", UNSET)
        for values_item_data in _values or []:
            values_item = swimlane_value_m.SwimlaneValue.from_dict(values_item_data)

            values.append(values_item)

        issue_based_swimlane_settings = cls(
            field=field,
            default_card_type=default_card_type,
            values=values,
            **_SwimlaneSettings_kwargs,
        )

        return issue_based_swimlane_settings



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="IssueLink")


@attr.s(auto_attribs=True)
class IssueLink:
    """Represents issue links of a particular link type (for example, 'relates to')."""

    direction: "Union[Unset, IssueLinkDirection]" = UNSET
    link_type: "Union[Unset, issue_link_type_m.IssueLinkType]" = UNSET
    issues: "Union[Unset, List[issue_m.Issue]]" = UNSET
    trimmed_issues: "Union[Unset, List[issue_m.Issue]]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        direction: Union[Unset, str] = UNSET
        if not isinstance(self.direction, Unset):
            direction = self.direction.value

        link_type: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.link_type, Unset):
            link_type = self.link_type.to_dict()

        issues: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.issues, Unset):
            issues = []
            for issues_item_data in self.issues:
                issues_item = issues_item_data.to_dict()

                issues.append(issues_item)

        trimmed_issues: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.trimmed_issues, Unset):
            trimmed_issues = []
            for trimmed_issues_item_data in self.trimmed_issues:
                trimmed_issues_item = trimmed_issues_item_data.to_dict()

                trimmed_issues.append(trimmed_issues_item)

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if direction is not UNSET:
            field_dict["direction"] = direction
        if link_type is not UNSET:
            field_dict["linkType"] = link_type
        if issues is not UNSET:
            field_dict["issues"] = issues
        if trimmed_issues is not UNSET:
            field_dict["trimmedIssues"] = trimmed_issues
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import issue as issue_m
            from ..models import issue_link_type as issue_link_type_m
            from ..models.issue_link_direction import IssueLinkDirection
        except ImportError:
            import sys

            issue_m = sys.modules[__package__ + "issue"]
            issue_link_type_m = sys.modules[__package__ + "issue_link_type"]
            IssueLinkDirection = sys.modules[__package__ + "IssueLinkDirecti"]

        d = src_dict.copy()

        _direction = d.pop("direction", UNSET)
        direction: Union[Unset, IssueLinkDirection]
        if isinstance(_direction, Unset):
            direction = UNSET
        else:
            direction = IssueLinkDirection(_direction)

        _link_type = d.pop("linkType", UNSET)
        link_type: Union[Unset, issue_link_type_m.IssueLinkType]
        if isinstance(_link_type, Unset):
            link_type = UNSET
        else:
            link_type = issue_link_type_m.IssueLinkType.from_dict(_link_type)

        issues = []
        _issues = d.pop("issues", UNSET)
        for issues_item_data in _issues or []:
            issues_item = issue_m.Issue.from_dict(issues_item_data)

            issues.append(issues_item)

        trimmed_issues = []
        _trimmed_issues = d.pop("trimmedIssues", UNSET)
        for trimmed_issues_item_data in _trimmed_issues or []:
            trimmed_issues_item = issue_m.Issue.from_dict(trimmed_issues_item_data)

            trimmed_issues.append(trimmed_issues_item)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        issue_link = cls(
            direction=direction,
            link_type=link_type,
            issues=issues,
            trimmed_issues=trimmed_issues,
            id=id,
            type=type,
        )

        return issue_link



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkTimeSettings")


@attr.s(auto_attribs=True)
class WorkTimeSettings:
    """Work schedule settings."""

    minutes_a_day: "Union[Unset, int]" = UNSET
    work_days: "Union[Unset, int]" = UNSET
    first_day_of_week: "Union[Unset, int]" = UNSET
    days_a_week: "Union[Unset, int]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        minutes_a_day = self.minutes_a_day
        work_days = self.work_days
        first_day_of_week = self.first_day_of_week
        days_a_week = self.days_a_week
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if minutes_a_day is not UNSET:
            field_dict["minutesADay"] = minutes_a_day
        if work_days is not UNSET:
            field_dict["workDays"] = work_days
        if first_day_of_week is not UNSET:
            field_dict["firstDayOfWeek"] = first_day_of_week
        if days_a_week is not UNSET:
            field_dict["daysAWeek"] = days_a_week
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        minutes_a_day = d.pop("minutesADay", UNSET)

        work_days = d.pop("workDays", UNSET)

        first_day_of_week = d.pop("firstDayOfWeek", UNSET)

        days_a_week = d.pop("daysAWeek", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        work_time_settings = cls(
            minutes_a_day=minutes_a_day,
            work_days=work_days,
            first_day_of_week=first_day_of_week,
            days_a_week=days_a_week,
            id=id,
            type=type,
        )

        return work_time_settings



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="IssueAttachment")


@attr.s(auto_attribs=True)
class IssueAttachment:
    """Represents a file that is attached to an issue or a comment."""

    name: "Union[Unset, str]" = UNSET
    author: "Union[Unset, user_m.User]" = UNSET
    created: "Union[Unset, int]" = UNSET
    updated: "Union[Unset, int]" = UNSET
    size: "Union[Unset, int]" = UNSET
    extension: "Union[Unset, str]" = UNSET
    charset: "Union[Unset, str]" = UNSET
    mime_type: "Union[Unset, str]" = UNSET
    meta_data: "Union[Unset, str]" = UNSET
    draft: "Union[Unset, bool]" = UNSET
    removed: "Union[Unset, bool]" = UNSET
    base_64_content: "Union[Unset, str]" = UNSET
    url: "Union[Unset, str]" = UNSET
    visibility: "Union[Unset, visibility_m.Visibility]" = UNSET
    issue: "Union[Unset, issue_m.Issue]" = UNSET
    comment: "Union[Unset, issue_comment_m.IssueComment]" = UNSET
    thumbnail_url: "Union[Unset, str]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        author: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.author, Unset):
            author = self.author.to_dict()

        created = self.created
        updated = self.updated
        size = self.size
        extension = self.extension
        charset = self.charset
        mime_type = self.mime_type
        meta_data = self.meta_data
        draft = self.draft
        removed = self.removed
        base_64_content = self.base_64_content
        url = self.url
        visibility: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.visibility, Unset):
            visibility = self.visibility.to_dict()

        issue: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.issue, Unset):
            issue = self.issue.to_dict()

        comment: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.comment, Unset):
            comment = self.comment.to_dict()

        thumbnail_url = self.thumbnail_url
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if author is not UNSET:
            field_dict["author"] = author
        if created is not UNSET:
            field_dict["created"] = created
        if updated is not UNSET:
            field_dict["updated"] = updated
        if size is not UNSET:
            field_dict["size"] = size
        if extension is not UNSET:
            field_dict["extension"] = extension
        if charset is not UNSET:
            field_dict["charset"] = charset
        if mime_type is not UNSET:
            field_dict["mimeType"] = mime_type
        if meta_data is not UNSET:
            field_dict["metaData"] = meta_data
        if draft is not UNSET:
            field_dict["draft"] = draft
        if removed is not UNSET:
            field_dict["removed"] = removed
        if base_64_content is not UNSET:
            field_dict["base64Content"] = base_64_content
        if url is not UNSET:
            field_dict["url"] = url
        if visibility is not UNSET:
            field_dict["visibility"] = visibility
        if issue is not UNSET:
            field_dict["issue"] = issue
        if comment is not UNSET:
            field_dict["comment"] = comment
        if thumbnail_url is not UNSET:
            field_dict["thumbnailURL"] = thumbnail_url
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import issue as issue_m
            from ..models import issue_comment as issue_comment_m
            from ..models import user as user_m
            from ..models import visibility as visibility_m
        except ImportError:
            import sys

            issue_m = sys.modules[__package__ + "issue"]
            issue_comment_m = sys.modules[__package__ + "issue_comment"]
            user_m = sys.modules[__package__ + "user"]
            visibility_m = sys.modules[__package__ + "visibility"]

        d = src_dict.copy()

        name = d.pop("name", UNSET)

        _author = d.pop("author", UNSET)
        author: Union[Unset, user_m.User]
        if isinstance(_author, Unset):
            author = UNSET
        else:
            author = user_m.User.from_dict(_author)

        created = d.pop("created", UNSET)

        updated = d.pop("updated", UNSET)

        size = d.pop("size", UNSET)

        extension = d.pop("extension", UNSET)

        charset = d.pop("charset", UNSET)

        mime_type = d.pop("mimeType", UNSET)

        meta_data = d.pop("metaData", UNSET)

        draft = d.pop("draft", UNSET)

        removed = d.pop("removed", UNSET)

        base_64_content = d.pop("base64Content", UNSET)

        url = d.pop("url", UNSET)

        _visibility = d.pop("visibility", UNSET)
        visibility: Union[Unset, visibility_m.Visibility]
        if isinstance(_visibility, Unset):
            visibility = UNSET
        else:
            visibility = visibility_m.Visibility.from_dict(_visibility)

        _issue = d.pop("issue", UNSET)
        issue: Union[Unset, issue_m.Issue]
        if isinstance(_issue, Unset):
            issue = UNSET
        else:
            issue = issue_m.Issue.from_dict(_issue)

        _comment = d.pop("comment", UNSET)
        comment: Union[Unset, issue_comment_m.IssueComment]
        if isinstance(_comment, Unset):
            comment = UNSET
        else:
            comment = issue_comment_m.IssueComment.from_dict(_comment)

        thumbnail_url = d.pop("thumbnailURL", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        issue_attachment = cls(
            name=name,
            author=author,
            created=created,
            updated=updated,
            size=size,
            extension=extension,
            charset=charset,
            mime_type=mime_type,
            meta_data=meta_data,
            draft=draft,
            removed=removed,
            base_64_content=base_64_content,
            url=url,
            visibility=visibility,
            issue=issue,
            comment=comment,
            thumbnail_url=thumbnail_url,
            id=id,
            type=type,
        )

        return issue_attachment



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CommandList")


@attr.s(auto_attribs=True)
class CommandList:
    """Represents list of command and related comment in YouTrack.
    Can be used to either apply a command or get command suggestions."""

    comment: "Union[Unset, str]" = UNSET
    visibility: "Union[Unset, command_visibility_m.CommandVisibility]" = UNSET
    query: "Union[Unset, str]" = UNSET
    caret: "Union[Unset, int]" = UNSET
    silent: "Union[Unset, bool]" = UNSET
    uses_markdown: "Union[Unset, bool]" = UNSET
    run_as: "Union[Unset, str]" = UNSET
    commands: "Union[Unset, List[parsed_command_m.ParsedCommand]]" = UNSET
    issues: "Union[Unset, List[issue_m.Issue]]" = UNSET
    suggestions: "Union[Unset, List[suggestion_m.Suggestion]]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        comment = self.comment
        visibility: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.visibility, Unset):
            visibility = self.visibility.to_dict()

        query = self.query
        caret = self.caret
        silent = self.silent
        uses_markdown = self.uses_markdown
        run_as = self.run_as
        commands: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.commands, Unset):
            commands = []
            for commands_item_data in self.commands:
                commands_item = commands_item_data.to_dict()

                commands.append(commands_item)

        issues: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.issues, Unset):
            issues = []
            for issues_item_data in self.issues:
                issues_item = issues_item_data.to_dict()

                issues.append(issues_item)

        suggestions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.suggestions, Unset):
            suggestions = []
            for suggestions_item_data in self.suggestions:
                suggestions_item = suggestions_item_data.to_dict()

                suggestions.append(suggestions_item)

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if comment is not UNSET:
            field_dict["comment"] = comment
        if visibility is not UNSET:
            field_dict["visibility"] = visibility
        if query is not UNSET:
            field_dict["query"] = query
        if caret is not UNSET:
            field_dict["caret"] = caret
        if silent is not UNSET:
            field_dict["silent"] = silent
        if uses_markdown is not UNSET:
            field_dict["usesMarkdown"] = uses_markdown
        if run_as is not UNSET:
            field_dict["runAs"] = run_as
        if commands is not UNSET:
            field_dict["commands"] = commands
        if issues is not UNSET:
            field_dict["issues"] = issues
        if suggestions is not UNSET:
            field_dict["suggestions"] = suggestions
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import command_visibility as command_visibility_m
            from ..models import issue as issue_m
            from ..models import parsed_command as parsed_command_m
            from ..models import suggestion as suggestion_m
        except ImportError:
            import sys

            issue_m = sys.modules[__package__ + "issue"]
            command_visibility_m = sys.modules[__package__ + "command_visibility"]
            parsed_command_m = sys.modules[__package__ + "parsed_command"]
            suggestion_m = sys.modules[__package__ + "suggestion"]

        d = src_dict.copy()

        comment = d.pop("comment", UNSET)

        _visibility = d.pop("visibility", UNSET)
        visibility: Union[Unset, command_visibility_m.CommandVisibility]
        if isinstance(_visibility, Unset):
            visibility = UNSET
        else:
            visibility = command_visibility_m.CommandVisibility.from_dict(_visibility)

        query = d.pop("query", UNSET)

        caret = d.pop("caret", UNSET)

        silent = d.pop("silent", UNSET)

        uses_markdown = d.pop("usesMarkdown", UNSET)

        run_as = d.pop("runAs", UNSET)

        commands = []
        _commands = d.pop("commands", UNSET)
        for commands_item_data in _commands or []:
            commands_item = parsed_command_m.ParsedCommand.from_dict(commands_item_data)

            commands.append(commands_item)

        issues = []
        _issues = d.pop("issues", UNSET)
        for issues_item_data in _issues or []:
            issues_item = issue_m.Issue.from_dict(issues_item_data)

            issues.append(issues_item)

        suggestions = []
        _suggestions = d.pop("suggestions", UNSET)
        for suggestions_item_data in _suggestions or []:
            suggestions_item = suggestion_m.Suggestion.from_dict(suggestions_item_data)

            suggestions.append(suggestions_item)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        command_list = cls(
            comment=comment,
            visibility=visibility,
            query=query,
            caret=caret,
            silent=silent,
            uses_markdown=uses_markdown,
            run_as=run_as,
            commands=commands,
            issues=issues,
            suggestions=suggestions,
            id=id,
            type=type,
        )

        return command_list



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ActivityCursorPage")


@attr.s(auto_attribs=True)
class ActivityCursorPage:
    """Represents a page object that wraps a list of issue activities.
    The main advantage of the page in comparision to a list of activities is cursors.
    The page provides boundary marks that allow continuous iteration over the activities from the place
    the page is finished."""

    reverse: "Union[Unset, bool]" = UNSET
    before_cursor: "Union[Unset, str]" = UNSET
    after_cursor: "Union[Unset, str]" = UNSET
    has_before: "Union[Unset, bool]" = UNSET
    has_after: "Union[Unset, bool]" = UNSET
    activities: "Union[Unset, List[activity_item_m.ActivityItem]]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        reverse = self.reverse
        before_cursor = self.before_cursor
        after_cursor = self.after_cursor
        has_before = self.has_before
        has_after = self.has_after
        activities: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.activities, Unset):
            activities = []
            for activities_item_data in self.activities:
                activities_item = activities_item_data.to_dict()

                activities.append(activities_item)

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if reverse is not UNSET:
            field_dict["reverse"] = reverse
        if before_cursor is not UNSET:
            field_dict["beforeCursor"] = before_cursor
        if after_cursor is not UNSET:
            field_dict["afterCursor"] = after_cursor
        if has_before is not UNSET:
            field_dict["hasBefore"] = has_before
        if has_after is not UNSET:
            field_dict["hasAfter"] = has_after
        if activities is not UNSET:
            field_dict["activities"] = activities
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import activity_item as activity_item_m
        except ImportError:
            import sys

            activity_item_m = sys.modules[__package__ + "activity_item"]

        d = src_dict.copy()

        reverse = d.pop("reverse", UNSET)

        before_cursor = d.pop("beforeCursor", UNSET)

        after_cursor = d.pop("afterCursor", UNSET)

        has_before = d.pop("hasBefore", UNSET)

        has_after = d.pop("hasAfter", UNSET)

        activities = []
        _activities = d.pop("activities", UNSET)
        for activities_item_data in _activities or []:
            activities_item = activity_item_m.ActivityItem.from_dict(activities_item_data)

            activities.append(activities_item)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        activity_cursor_page = cls(
            reverse=reverse,
            before_cursor=before_cursor,
            after_cursor=after_cursor,
            has_before=has_before,
            has_after=has_after,
            activities=activities,
            id=id,
            type=type,
        )

        return activity_cursor_page



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="SingleVersionIssueCustomField")


try:
    from ..models import database_single_value_issue_custom_field
except ImportError:
    import sys

    database_single_value_issue_custom_field = sys.modules[__package__ + "database_single_value_issue_custom_field"]


@attr.s(auto_attribs=True)
class SingleVersionIssueCustomField(database_single_value_issue_custom_field.DatabaseSingleValueIssueCustomField):
    """Represents the issue custom field of the `version` type that can only have a single value."""

    value: "Union[Unset, version_bundle_element_m.VersionBundleElement]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        value: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.value, Unset):
            value = self.value.to_dict()

        field_dict: Dict[str, Any] = {}
        _DatabaseSingleValueIssueCustomField_dict = super().to_dict()
        field_dict.update(_DatabaseSingleValueIssueCustomField_dict)
        field_dict.update({})
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import version_bundle_element as version_bundle_element_m
        except ImportError:
            import sys

            version_bundle_element_m = sys.modules[__package__ + "version_bundle_element"]

        d = src_dict.copy()

        _DatabaseSingleValueIssueCustomField_kwargs = super().from_dict(src_dict=d).to_dict()
        _DatabaseSingleValueIssueCustomField_kwargs.pop("$type")

        _value = d.pop("value", UNSET)
        value: Union[Unset, version_bundle_element_m.VersionBundleElement]
        if isinstance(_value, Unset):
            value = UNSET
        else:
            value = version_bundle_element_m.VersionBundleElement.from_dict(_value)

        single_version_issue_custom_field = cls(
            value=value,
            **_DatabaseSingleValueIssueCustomField_kwargs,
        )

        return single_version_issue_custom_field



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CustomFieldDefaults")


@attr.s(auto_attribs=True)
class CustomFieldDefaults:
    """Represents default project-related settings of the custom field. These settings are applied at the moment when the custom field is attached to a project. After that, any changes in default settings do not affect the field settings for this project."""

    can_be_empty: "Union[Unset, bool]" = UNSET
    empty_field_text: "Union[Unset, str]" = UNSET
    is_public: "Union[Unset, bool]" = UNSET
    parent: "Union[Unset, custom_field_m.CustomField]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        can_be_empty = self.can_be_empty
        empty_field_text = self.empty_field_text
        is_public = self.is_public
        parent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parent, Unset):
            parent = self.parent.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if can_be_empty is not UNSET:
            field_dict["canBeEmpty"] = can_be_empty
        if empty_field_text is not UNSET:
            field_dict["emptyFieldText"] = empty_field_text
        if is_public is not UNSET:
            field_dict["isPublic"] = is_public
        if parent is not UNSET:
            field_dict["parent"] = parent
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import custom_field as custom_field_m
        except ImportError:
            import sys

            custom_field_m = sys.modules[__package__ + "custom_field"]

        d = src_dict.copy()

        can_be_empty = d.pop("canBeEmpty", UNSET)

        empty_field_text = d.pop("emptyFieldText", UNSET)

        is_public = d.pop("isPublic", UNSET)

        _parent = d.pop("parent", UNSET)
        parent: Union[Unset, custom_field_m.CustomField]
        if isinstance(_parent, Unset):
            parent = UNSET
        else:
            parent = custom_field_m.CustomField.from_dict(_parent)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        custom_field_defaults = cls(
            can_be_empty=can_be_empty,
            empty_field_text=empty_field_text,
            is_public=is_public,
            parent=parent,
            id=id,
            type=type,
        )

        return custom_field_defaults



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EmailSettings")


@attr.s(auto_attribs=True)
class EmailSettings:
    """Represents email settings for this YouTrack installation."""

    is_enabled: "Union[Unset, bool]" = UNSET
    host: "Union[Unset, str]" = UNSET
    port: "Union[Unset, int]" = UNSET
    mail_protocol: "Union[Unset, EmailSettingsMailProtocol]" = UNSET
    anonymous: "Union[Unset, bool]" = UNSET
    login: "Union[Unset, str]" = UNSET
    ssl_key: "Union[Unset, storage_entry_m.StorageEntry]" = UNSET
    from_: "Union[Unset, str]" = UNSET
    reply_to: "Union[Unset, str]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        is_enabled = self.is_enabled
        host = self.host
        port = self.port
        mail_protocol: Union[Unset, str] = UNSET
        if not isinstance(self.mail_protocol, Unset):
            mail_protocol = self.mail_protocol.value

        anonymous = self.anonymous
        login = self.login
        ssl_key: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.ssl_key, Unset):
            ssl_key = self.ssl_key.to_dict()

        from_ = self.from_
        reply_to = self.reply_to
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if is_enabled is not UNSET:
            field_dict["isEnabled"] = is_enabled
        if host is not UNSET:
            field_dict["host"] = host
        if port is not UNSET:
            field_dict["port"] = port
        if mail_protocol is not UNSET:
            field_dict["mailProtocol"] = mail_protocol
        if anonymous is not UNSET:
            field_dict["anonymous"] = anonymous
        if login is not UNSET:
            field_dict["login"] = login
        if ssl_key is not UNSET:
            field_dict["sslKey"] = ssl_key
        if from_ is not UNSET:
            field_dict["from"] = from_
        if reply_to is not UNSET:
            field_dict["replyTo"] = reply_to
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import storage_entry as storage_entry_m
            from ..models.email_settings_mail_protocol import EmailSettingsMailProtocol
        except ImportError:
            import sys

            storage_entry_m = sys.modules[__package__ + "storage_entry"]
            EmailSettingsMailProtocol = sys.modules[__package__ + "EmailSettingsMailProtoc"]

        d = src_dict.copy()

        is_enabled = d.pop("isEnabled", UNSET)

        host = d.pop("host", UNSET)

        port = d.pop("port", UNSET)

        _mail_protocol = d.pop("mailProtocol", UNSET)
        mail_protocol: Union[Unset, EmailSettingsMailProtocol]
        if isinstance(_mail_protocol, Unset):
            mail_protocol = UNSET
        else:
            mail_protocol = EmailSettingsMailProtocol(_mail_protocol)

        anonymous = d.pop("anonymous", UNSET)

        login = d.pop("login", UNSET)

        _ssl_key = d.pop("sslKey", UNSET)
        ssl_key: Union[Unset, storage_entry_m.StorageEntry]
        if isinstance(_ssl_key, Unset):
            ssl_key = UNSET
        else:
            ssl_key = storage_entry_m.StorageEntry.from_dict(_ssl_key)

        from_ = d.pop("from", UNSET)

        reply_to = d.pop("replyTo", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        email_settings = cls(
            is_enabled=is_enabled,
            host=host,
            port=port,
            mail_protocol=mail_protocol,
            anonymous=anonymous,
            login=login,
            ssl_key=ssl_key,
            from_=from_,
            reply_to=reply_to,
            id=id,
            type=type,
        )

        return email_settings



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="SystemSettings")


@attr.s(auto_attribs=True)
class SystemSettings:
    """Represents the System settings that affect core functionality of YouTrack."""

    base_url: "Union[Unset, str]" = UNSET
    max_upload_file_size: "Union[Unset, int]" = UNSET
    max_export_items: "Union[Unset, int]" = UNSET
    administrator_email: "Union[Unset, str]" = UNSET
    allow_statistics_collection: "Union[Unset, bool]" = UNSET
    is_application_read_only: "Union[Unset, bool]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        base_url = self.base_url
        max_upload_file_size = self.max_upload_file_size
        max_export_items = self.max_export_items
        administrator_email = self.administrator_email
        allow_statistics_collection = self.allow_statistics_collection
        is_application_read_only = self.is_application_read_only
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if base_url is not UNSET:
            field_dict["baseUrl"] = base_url
        if max_upload_file_size is not UNSET:
            field_dict["maxUploadFileSize"] = max_upload_file_size
        if max_export_items is not UNSET:
            field_dict["maxExportItems"] = max_export_items
        if administrator_email is not UNSET:
            field_dict["administratorEmail"] = administrator_email
        if allow_statistics_collection is not UNSET:
            field_dict["allowStatisticsCollection"] = allow_statistics_collection
        if is_application_read_only is not UNSET:
            field_dict["isApplicationReadOnly"] = is_application_read_only
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        base_url = d.pop("baseUrl", UNSET)

        max_upload_file_size = d.pop("maxUploadFileSize", UNSET)

        max_export_items = d.pop("maxExportItems", UNSET)

        administrator_email = d.pop("administratorEmail", UNSET)

        allow_statistics_collection = d.pop("allowStatisticsCollection", UNSET)

        is_application_read_only = d.pop("isApplicationReadOnly", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        system_settings = cls(
            base_url=base_url,
            max_upload_file_size=max_upload_file_size,
            max_export_items=max_export_items,
            administrator_email=administrator_email,
            allow_statistics_collection=allow_statistics_collection,
            is_application_read_only=is_application_read_only,
            id=id,
            type=type,
        )

        return system_settings



""" Contains all the data models used in inputs/outputs """

from .activity_category import ActivityCategory
from .activity_cursor_page import ActivityCursorPage
from .activity_item import ActivityItem
from .agile import Agile
from .agile_column import AgileColumn
from .agile_column_field_value import AgileColumnFieldValue
from .agile_status import AgileStatus
from .appearance_settings import AppearanceSettings
from .attachment_activity_item import AttachmentActivityItem
from .attribute_based_swimlane_settings import AttributeBasedSwimlaneSettings
from .backup_error import BackupError
from .backup_file import BackupFile
from .backup_status import BackupStatus
from .base_bundle import BaseBundle
from .build_bundle import BuildBundle
from .build_bundle_custom_field_defaults import BuildBundleCustomFieldDefaults
from .build_bundle_element import BuildBundleElement
from .build_project_custom_field import BuildProjectCustomField
from .bundle import Bundle
from .bundle_custom_field_defaults import BundleCustomFieldDefaults
from .bundle_element import BundleElement
from .bundle_project_custom_field import BundleProjectCustomField
from .color_coding import ColorCoding
from .column_settings import ColumnSettings
from .command_limited_visibility import CommandLimitedVisibility
from .command_list import CommandList
from .command_unlimited_visibility import CommandUnlimitedVisibility
from .command_visibility import CommandVisibility
from .comment_activity_item import CommentActivityItem
from .comment_attachments_activity_item import CommentAttachmentsActivityItem
from .created_deleted_activity_item import CreatedDeletedActivityItem
from .custom_field import CustomField
from .custom_field_activity_item import CustomFieldActivityItem
from .custom_field_activity_item_added import CustomFieldActivityItemAdded
from .custom_field_activity_item_removed import CustomFieldActivityItemRemoved
from .custom_field_defaults import CustomFieldDefaults
from .custom_filter_field import CustomFilterField
from .database_attribute_value import DatabaseAttributeValue
from .database_backup_settings import DatabaseBackupSettings
from .database_backup_settings_archive_format import DatabaseBackupSettingsArchiveFormat
from .database_multi_value_issue_custom_field import DatabaseMultiValueIssueCustomField
from .database_multi_value_issue_custom_field_value import DatabaseMultiValueIssueCustomFieldValue
from .database_single_value_issue_custom_field import DatabaseSingleValueIssueCustomField
from .database_single_value_issue_custom_field_value import DatabaseSingleValueIssueCustomFieldValue
from .date_format_descriptor import DateFormatDescriptor
from .date_issue_custom_field import DateIssueCustomField
from .date_issue_custom_field_value import DateIssueCustomFieldValue
from .duplicate_vote import DuplicateVote
from .duration_value import DurationValue
from .email_settings import EmailSettings
from .email_settings_mail_protocol import EmailSettingsMailProtocol
from .enum_bundle import EnumBundle
from .enum_bundle_custom_field_defaults import EnumBundleCustomFieldDefaults
from .enum_bundle_element import EnumBundleElement
from .enum_project_custom_field import EnumProjectCustomField
from .external_issue import ExternalIssue
from .field_based_color_coding import FieldBasedColorCoding
from .field_style import FieldStyle
from .field_type import FieldType
from .filter_field import FilterField
from .general_user_profile import GeneralUserProfile
from .global_settings import GlobalSettings
from .global_time_tracking_settings import GlobalTimeTrackingSettings
from .group_project_custom_field import GroupProjectCustomField
from .issue import Issue
from .issue_attachment import IssueAttachment
from .issue_based_swimlane_settings import IssueBasedSwimlaneSettings
from .issue_comment import IssueComment
from .issue_created_activity_item import IssueCreatedActivityItem
from .issue_custom_field import IssueCustomField
from .issue_folder import IssueFolder
from .issue_link import IssueLink
from .issue_link_direction import IssueLinkDirection
from .issue_link_type import IssueLinkType
from .issue_resolved_activity_item import IssueResolvedActivityItem
from .issue_tag import IssueTag
from .issue_time_tracker import IssueTimeTracker
from .issue_voters import IssueVoters
from .issue_watcher import IssueWatcher
from .issue_watchers import IssueWatchers
from .issue_work_item import IssueWorkItem
from .jabber_settings import JabberSettings
from .license_ import License
from .limited_visibility import LimitedVisibility
from .links_activity_item import LinksActivityItem
from .locale_descriptor import LocaleDescriptor
from .locale_settings import LocaleSettings
from .localizable_bundle_element import LocalizableBundleElement
from .logo import Logo
from .me import Me
from .multi_build_issue_custom_field import MultiBuildIssueCustomField
from .multi_enum_issue_custom_field import MultiEnumIssueCustomField
from .multi_group_issue_custom_field import MultiGroupIssueCustomField
from .multi_owned_issue_custom_field import MultiOwnedIssueCustomField
from .multi_user_issue_custom_field import MultiUserIssueCustomField
from .multi_value_activity_item import MultiValueActivityItem
from .multi_version_issue_custom_field import MultiVersionIssueCustomField
from .notification_settings import NotificationSettings
from .notifications_user_profile import NotificationsUserProfile
from .online_users import OnlineUsers
from .owned_bundle import OwnedBundle
from .owned_bundle_custom_field_defaults import OwnedBundleCustomFieldDefaults
from .owned_bundle_element import OwnedBundleElement
from .owned_project_custom_field import OwnedProjectCustomField
from .parsed_command import ParsedCommand
from .period_field_format import PeriodFieldFormat
from .period_issue_custom_field import PeriodIssueCustomField
from .period_project_custom_field import PeriodProjectCustomField
from .period_value import PeriodValue
from .predefined_filter_field import PredefinedFilterField
from .project import Project
from .project_activity_item import ProjectActivityItem
from .project_activity_item_added import ProjectActivityItemAdded
from .project_activity_item_removed import ProjectActivityItemRemoved
from .project_based_color_coding import ProjectBasedColorCoding
from .project_color import ProjectColor
from .project_custom_field import ProjectCustomField
from .project_custom_fields import ProjectCustomFields
from .project_time_tracking_settings import ProjectTimeTrackingSettings
from .rest_cors_settings import RestCorsSettings
from .saved_query import SavedQuery
from .search_suggestions import SearchSuggestions
from .simple_issue_custom_field import SimpleIssueCustomField
from .simple_issue_custom_field_value import SimpleIssueCustomFieldValue
from .simple_project_custom_field import SimpleProjectCustomField
from .simple_value_activity_item import SimpleValueActivityItem
from .simple_value_activity_item_added import SimpleValueActivityItemAdded
from .simple_value_activity_item_removed import SimpleValueActivityItemRemoved
from .single_build_issue_custom_field import SingleBuildIssueCustomField
from .single_enum_issue_custom_field import SingleEnumIssueCustomField
from .single_group_issue_custom_field import SingleGroupIssueCustomField
from .single_owned_issue_custom_field import SingleOwnedIssueCustomField
from .single_user_issue_custom_field import SingleUserIssueCustomField
from .single_value_activity_item import SingleValueActivityItem
from .single_version_issue_custom_field import SingleVersionIssueCustomField
from .sprint import Sprint
from .sprint_activity_item import SprintActivityItem
from .sprints_settings import SprintsSettings
from .state_bundle import StateBundle
from .state_bundle_custom_field_defaults import StateBundleCustomFieldDefaults
from .state_bundle_element import StateBundleElement
from .state_issue_custom_field import StateIssueCustomField
from .state_project_custom_field import StateProjectCustomField
from .storage_entry import StorageEntry
from .suggestion import Suggestion
from .swimlane_entity_attribute_value import SwimlaneEntityAttributeValue
from .swimlane_settings import SwimlaneSettings
from .swimlane_value import SwimlaneValue
from .system_settings import SystemSettings
from .tags_activity_item import TagsActivityItem
from .telemetry import Telemetry
from .text_custom_field_activity_item import TextCustomFieldActivityItem
from .text_field_value import TextFieldValue
from .text_issue_custom_field import TextIssueCustomField
from .text_markup_activity_item import TextMarkupActivityItem
from .text_project_custom_field import TextProjectCustomField
from .time_tracking_user_profile import TimeTrackingUserProfile
from .time_zone_descriptor import TimeZoneDescriptor
from .unlimited_visibility import UnlimitedVisibility
from .user import User
from .user_bundle import UserBundle
from .user_custom_field_defaults import UserCustomFieldDefaults
from .user_group import UserGroup
from .user_profiles import UserProfiles
from .user_project_custom_field import UserProjectCustomField
from .uses_markup_activity_item import UsesMarkupActivityItem
from .vcs_change_activity_item import VcsChangeActivityItem
from .vcs_change_activity_item_added import VcsChangeActivityItemAdded
from .vcs_change_activity_item_removed import VcsChangeActivityItemRemoved
from .vcs_unresolved_user import VcsUnresolvedUser
from .version_bundle import VersionBundle
from .version_bundle_custom_field_defaults import VersionBundleCustomFieldDefaults
from .version_bundle_element import VersionBundleElement
from .version_project_custom_field import VersionProjectCustomField
from .visibility import Visibility
from .visibility_activity_item import VisibilityActivityItem
from .visibility_group_activity_item import VisibilityGroupActivityItem
from .visibility_user_activity_item import VisibilityUserActivityItem
from .voters_activity_item import VotersActivityItem
from .watch_folder import WatchFolder
from .wip_limit import WIPLimit
from .work_item_activity_item import WorkItemActivityItem
from .work_item_author_activity_item import WorkItemAuthorActivityItem
from .work_item_duration_activity_item import WorkItemDurationActivityItem
from .work_item_type import WorkItemType
from .work_item_type_activity_item import WorkItemTypeActivityItem
from .work_time_settings import WorkTimeSettings



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="OwnedBundleElement")


try:
    from ..models import bundle_element
except ImportError:
    import sys

    bundle_element = sys.modules[__package__ + "bundle_element"]


@attr.s(auto_attribs=True)
class OwnedBundleElement(bundle_element.BundleElement):
    """Represents a single owned value of a set. For example, a subsystem."""

    owner: "Union[Unset, user_m.User]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        owner: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner, Unset):
            owner = self.owner.to_dict()

        field_dict: Dict[str, Any] = {}
        _BundleElement_dict = super().to_dict()
        field_dict.update(_BundleElement_dict)
        field_dict.update({})
        if owner is not UNSET:
            field_dict["owner"] = owner

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import user as user_m
        except ImportError:
            import sys

            user_m = sys.modules[__package__ + "user"]

        d = src_dict.copy()

        _BundleElement_kwargs = super().from_dict(src_dict=d).to_dict()
        _BundleElement_kwargs.pop("$type")

        _owner = d.pop("owner", UNSET)
        owner: Union[Unset, user_m.User]
        if isinstance(_owner, Unset):
            owner = UNSET
        else:
            owner = user_m.User.from_dict(_owner)

        owned_bundle_element = cls(
            owner=owner,
            **_BundleElement_kwargs,
        )

        return owned_bundle_element



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UserBundle")


try:
    from ..models import bundle
except ImportError:
    import sys

    bundle = sys.modules[__package__ + "bundle"]


@attr.s(auto_attribs=True)
class UserBundle(bundle.Bundle):
    """Represents a set of values that contains users. You can add to the set both individual user accounts and groups of users."""

    groups: "Union[Unset, List[user_group_m.UserGroup]]" = UNSET
    individuals: "Union[Unset, List[user_m.User]]" = UNSET
    aggregated_users: "Union[Unset, List[user_m.User]]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        groups: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.groups, Unset):
            groups = []
            for groups_item_data in self.groups:
                groups_item = groups_item_data.to_dict()

                groups.append(groups_item)

        individuals: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.individuals, Unset):
            individuals = []
            for individuals_item_data in self.individuals:
                individuals_item = individuals_item_data.to_dict()

                individuals.append(individuals_item)

        aggregated_users: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.aggregated_users, Unset):
            aggregated_users = []
            for aggregated_users_item_data in self.aggregated_users:
                aggregated_users_item = aggregated_users_item_data.to_dict()

                aggregated_users.append(aggregated_users_item)

        field_dict: Dict[str, Any] = {}
        _Bundle_dict = super().to_dict()
        field_dict.update(_Bundle_dict)
        field_dict.update({})
        if groups is not UNSET:
            field_dict["groups"] = groups
        if individuals is not UNSET:
            field_dict["individuals"] = individuals
        if aggregated_users is not UNSET:
            field_dict["aggregatedUsers"] = aggregated_users

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import user as user_m
            from ..models import user_group as user_group_m
        except ImportError:
            import sys

            user_group_m = sys.modules[__package__ + "user_group"]
            user_m = sys.modules[__package__ + "user"]

        d = src_dict.copy()

        _Bundle_kwargs = super().from_dict(src_dict=d).to_dict()
        _Bundle_kwargs.pop("$type")

        groups = []
        _groups = d.pop("groups", UNSET)
        for groups_item_data in _groups or []:
            groups_item = user_group_m.UserGroup.from_dict(groups_item_data)

            groups.append(groups_item)

        individuals = []
        _individuals = d.pop("individuals", UNSET)
        for individuals_item_data in _individuals or []:
            individuals_item = user_m.User.from_dict(individuals_item_data)

            individuals.append(individuals_item)

        aggregated_users = []
        _aggregated_users = d.pop("aggregatedUsers", UNSET)
        for aggregated_users_item_data in _aggregated_users or []:
            aggregated_users_item = user_m.User.from_dict(aggregated_users_item_data)

            aggregated_users.append(aggregated_users_item)

        user_bundle = cls(
            groups=groups,
            individuals=individuals,
            aggregated_users=aggregated_users,
            **_Bundle_kwargs,
        )

        return user_bundle



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GeneralUserProfile")


@attr.s(auto_attribs=True)
class GeneralUserProfile:
    """Represents the user profile in YouTrack."""

    date_field_format: "Union[Unset, date_format_descriptor_m.DateFormatDescriptor]" = UNSET
    timezone: "Union[Unset, time_zone_descriptor_m.TimeZoneDescriptor]" = UNSET
    locale: "Union[Unset, locale_descriptor_m.LocaleDescriptor]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        date_field_format: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.date_field_format, Unset):
            date_field_format = self.date_field_format.to_dict()

        timezone: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.timezone, Unset):
            timezone = self.timezone.to_dict()

        locale: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.locale, Unset):
            locale = self.locale.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if date_field_format is not UNSET:
            field_dict["dateFieldFormat"] = date_field_format
        if timezone is not UNSET:
            field_dict["timezone"] = timezone
        if locale is not UNSET:
            field_dict["locale"] = locale
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import date_format_descriptor as date_format_descriptor_m
            from ..models import locale_descriptor as locale_descriptor_m
            from ..models import time_zone_descriptor as time_zone_descriptor_m
        except ImportError:
            import sys

            time_zone_descriptor_m = sys.modules[__package__ + "time_zone_descriptor"]
            locale_descriptor_m = sys.modules[__package__ + "locale_descriptor"]
            date_format_descriptor_m = sys.modules[__package__ + "date_format_descriptor"]

        d = src_dict.copy()

        _date_field_format = d.pop("dateFieldFormat", UNSET)
        date_field_format: Union[Unset, date_format_descriptor_m.DateFormatDescriptor]
        if isinstance(_date_field_format, Unset):
            date_field_format = UNSET
        else:
            date_field_format = date_format_descriptor_m.DateFormatDescriptor.from_dict(_date_field_format)

        _timezone = d.pop("timezone", UNSET)
        timezone: Union[Unset, time_zone_descriptor_m.TimeZoneDescriptor]
        if isinstance(_timezone, Unset):
            timezone = UNSET
        else:
            timezone = time_zone_descriptor_m.TimeZoneDescriptor.from_dict(_timezone)

        _locale = d.pop("locale", UNSET)
        locale: Union[Unset, locale_descriptor_m.LocaleDescriptor]
        if isinstance(_locale, Unset):
            locale = UNSET
        else:
            locale = locale_descriptor_m.LocaleDescriptor.from_dict(_locale)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        general_user_profile = cls(
            date_field_format=date_field_format,
            timezone=timezone,
            locale=locale,
            id=id,
            type=type,
        )

        return general_user_profile



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="BackupStatus")


@attr.s(auto_attribs=True)
class BackupStatus:
    """Represents the current status of the backup process."""

    backup_in_progress: "Union[Unset, bool]" = UNSET
    backup_cancelled: "Union[Unset, bool]" = UNSET
    backup_error: "Union[Unset, backup_error_m.BackupError]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        backup_in_progress = self.backup_in_progress
        backup_cancelled = self.backup_cancelled
        backup_error: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.backup_error, Unset):
            backup_error = self.backup_error.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if backup_in_progress is not UNSET:
            field_dict["backupInProgress"] = backup_in_progress
        if backup_cancelled is not UNSET:
            field_dict["backupCancelled"] = backup_cancelled
        if backup_error is not UNSET:
            field_dict["backupError"] = backup_error
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import backup_error as backup_error_m
        except ImportError:
            import sys

            backup_error_m = sys.modules[__package__ + "backup_error"]

        d = src_dict.copy()

        backup_in_progress = d.pop("backupInProgress", UNSET)

        backup_cancelled = d.pop("backupCancelled", UNSET)

        _backup_error = d.pop("backupError", UNSET)
        backup_error: Union[Unset, backup_error_m.BackupError]
        if isinstance(_backup_error, Unset):
            backup_error = UNSET
        else:
            backup_error = backup_error_m.BackupError.from_dict(_backup_error)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        backup_status = cls(
            backup_in_progress=backup_in_progress,
            backup_cancelled=backup_cancelled,
            backup_error=backup_error,
            id=id,
            type=type,
        )

        return backup_status



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="License")


@attr.s(auto_attribs=True)
class License:
    """Represents the current license of the YouTrack service."""

    username: "Union[Unset, str]" = UNSET
    license_: "Union[Unset, str]" = UNSET
    error: "Union[Unset, str]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        username = self.username
        license_ = self.license_
        error = self.error
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if username is not UNSET:
            field_dict["username"] = username
        if license_ is not UNSET:
            field_dict["license"] = license_
        if error is not UNSET:
            field_dict["error"] = error
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        username = d.pop("username", UNSET)

        license_ = d.pop("license", UNSET)

        error = d.pop("error", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        license_ = cls(
            username=username,
            license_=license_,
            error=error,
            id=id,
            type=type,
        )

        return license_



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Issue")


@attr.s(auto_attribs=True)
class Issue:
    """Represents an issue in YouTrack."""

    id_readable: "Union[Unset, str]" = UNSET
    created: "Union[Unset, int]" = UNSET
    updated: "Union[Unset, int]" = UNSET
    resolved: "Union[Unset, int]" = UNSET
    number_in_project: "Union[Unset, int]" = UNSET
    project: "Union[Unset, project_m.Project]" = UNSET
    summary: "Union[Unset, str]" = UNSET
    description: "Union[Unset, str]" = UNSET
    uses_markdown: "Union[Unset, bool]" = UNSET
    wikified_description: "Union[Unset, str]" = UNSET
    reporter: "Union[Unset, user_m.User]" = UNSET
    updater: "Union[Unset, user_m.User]" = UNSET
    draft_owner: "Union[Unset, user_m.User]" = UNSET
    is_draft: "Union[Unset, bool]" = UNSET
    visibility: "Union[Unset, visibility_m.Visibility]" = UNSET
    votes: "Union[Unset, int]" = UNSET
    comments: "Union[Unset, List[issue_comment_m.IssueComment]]" = UNSET
    comments_count: "Union[Unset, int]" = UNSET
    tags: "Union[Unset, List[issue_tag_m.IssueTag]]" = UNSET
    links: "Union[Unset, List[issue_link_m.IssueLink]]" = UNSET
    external_issue: "Union[Unset, external_issue_m.ExternalIssue]" = UNSET
    custom_fields: "Union[Unset, List[issue_custom_field_m.IssueCustomField]]" = UNSET
    voters: "Union[Unset, issue_voters_m.IssueVoters]" = UNSET
    watchers: "Union[Unset, issue_watchers_m.IssueWatchers]" = UNSET
    attachments: "Union[Unset, List[issue_attachment_m.IssueAttachment]]" = UNSET
    subtasks: "Union[Unset, issue_link_m.IssueLink]" = UNSET
    parent: "Union[Unset, issue_link_m.IssueLink]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id_readable = self.id_readable
        created = self.created
        updated = self.updated
        resolved = self.resolved
        number_in_project = self.number_in_project
        project: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.project, Unset):
            project = self.project.to_dict()

        summary = self.summary
        description = self.description
        uses_markdown = self.uses_markdown
        wikified_description = self.wikified_description
        reporter: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.reporter, Unset):
            reporter = self.reporter.to_dict()

        updater: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.updater, Unset):
            updater = self.updater.to_dict()

        draft_owner: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.draft_owner, Unset):
            draft_owner = self.draft_owner.to_dict()

        is_draft = self.is_draft
        visibility: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.visibility, Unset):
            visibility = self.visibility.to_dict()

        votes = self.votes
        comments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.comments, Unset):
            comments = []
            for comments_item_data in self.comments:
                comments_item = comments_item_data.to_dict()

                comments.append(comments_item)

        comments_count = self.comments_count
        tags: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = []
            for tags_item_data in self.tags:
                tags_item = tags_item_data.to_dict()

                tags.append(tags_item)

        links: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.links, Unset):
            links = []
            for links_item_data in self.links:
                links_item = links_item_data.to_dict()

                links.append(links_item)

        external_issue: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.external_issue, Unset):
            external_issue = self.external_issue.to_dict()

        custom_fields: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.custom_fields, Unset):
            custom_fields = []
            for custom_fields_item_data in self.custom_fields:
                custom_fields_item = custom_fields_item_data.to_dict()

                custom_fields.append(custom_fields_item)

        voters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.voters, Unset):
            voters = self.voters.to_dict()

        watchers: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.watchers, Unset):
            watchers = self.watchers.to_dict()

        attachments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.attachments, Unset):
            attachments = []
            for attachments_item_data in self.attachments:
                attachments_item = attachments_item_data.to_dict()

                attachments.append(attachments_item)

        subtasks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.subtasks, Unset):
            subtasks = self.subtasks.to_dict()

        parent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parent, Unset):
            parent = self.parent.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if id_readable is not UNSET:
            field_dict["idReadable"] = id_readable
        if created is not UNSET:
            field_dict["created"] = created
        if updated is not UNSET:
            field_dict["updated"] = updated
        if resolved is not UNSET:
            field_dict["resolved"] = resolved
        if number_in_project is not UNSET:
            field_dict["numberInProject"] = number_in_project
        if project is not UNSET:
            field_dict["project"] = project
        if summary is not UNSET:
            field_dict["summary"] = summary
        if description is not UNSET:
            field_dict["description"] = description
        if uses_markdown is not UNSET:
            field_dict["usesMarkdown"] = uses_markdown
        if wikified_description is not UNSET:
            field_dict["wikifiedDescription"] = wikified_description
        if reporter is not UNSET:
            field_dict["reporter"] = reporter
        if updater is not UNSET:
            field_dict["updater"] = updater
        if draft_owner is not UNSET:
            field_dict["draftOwner"] = draft_owner
        if is_draft is not UNSET:
            field_dict["isDraft"] = is_draft
        if visibility is not UNSET:
            field_dict["visibility"] = visibility
        if votes is not UNSET:
            field_dict["votes"] = votes
        if comments is not UNSET:
            field_dict["comments"] = comments
        if comments_count is not UNSET:
            field_dict["commentsCount"] = comments_count
        if tags is not UNSET:
            field_dict["tags"] = tags
        if links is not UNSET:
            field_dict["links"] = links
        if external_issue is not UNSET:
            field_dict["externalIssue"] = external_issue
        if custom_fields is not UNSET:
            field_dict["customFields"] = custom_fields
        if voters is not UNSET:
            field_dict["voters"] = voters
        if watchers is not UNSET:
            field_dict["watchers"] = watchers
        if attachments is not UNSET:
            field_dict["attachments"] = attachments
        if subtasks is not UNSET:
            field_dict["subtasks"] = subtasks
        if parent is not UNSET:
            field_dict["parent"] = parent
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import external_issue as external_issue_m
            from ..models import issue_attachment as issue_attachment_m
            from ..models import issue_comment as issue_comment_m
            from ..models import issue_custom_field as issue_custom_field_m
            from ..models import issue_link as issue_link_m
            from ..models import issue_tag as issue_tag_m
            from ..models import issue_voters as issue_voters_m
            from ..models import issue_watchers as issue_watchers_m
            from ..models import project as project_m
            from ..models import user as user_m
            from ..models import visibility as visibility_m
        except ImportError:
            import sys

            issue_link_m = sys.modules[__package__ + "issue_link"]
            project_m = sys.modules[__package__ + "project"]
            issue_watchers_m = sys.modules[__package__ + "issue_watchers"]
            issue_comment_m = sys.modules[__package__ + "issue_comment"]
            issue_tag_m = sys.modules[__package__ + "issue_tag"]
            issue_custom_field_m = sys.modules[__package__ + "issue_custom_field"]
            user_m = sys.modules[__package__ + "user"]
            issue_attachment_m = sys.modules[__package__ + "issue_attachment"]
            external_issue_m = sys.modules[__package__ + "external_issue"]
            visibility_m = sys.modules[__package__ + "visibility"]
            issue_voters_m = sys.modules[__package__ + "issue_voters"]

        d = src_dict.copy()

        id_readable = d.pop("idReadable", UNSET)

        created = d.pop("created", UNSET)

        updated = d.pop("updated", UNSET)

        resolved = d.pop("resolved", UNSET)

        number_in_project = d.pop("numberInProject", UNSET)

        _project = d.pop("project", UNSET)
        project: Union[Unset, project_m.Project]
        if isinstance(_project, Unset):
            project = UNSET
        else:
            project = project_m.Project.from_dict(_project)

        summary = d.pop("summary", UNSET)

        description = d.pop("description", UNSET)

        uses_markdown = d.pop("usesMarkdown", UNSET)

        wikified_description = d.pop("wikifiedDescription", UNSET)

        _reporter = d.pop("reporter", UNSET)
        reporter: Union[Unset, user_m.User]
        if isinstance(_reporter, Unset):
            reporter = UNSET
        else:
            reporter = user_m.User.from_dict(_reporter)

        _updater = d.pop("updater", UNSET)
        updater: Union[Unset, user_m.User]
        if isinstance(_updater, Unset):
            updater = UNSET
        else:
            updater = user_m.User.from_dict(_updater)

        _draft_owner = d.pop("draftOwner", UNSET)
        draft_owner: Union[Unset, user_m.User]
        if isinstance(_draft_owner, Unset):
            draft_owner = UNSET
        else:
            draft_owner = user_m.User.from_dict(_draft_owner)

        is_draft = d.pop("isDraft", UNSET)

        _visibility = d.pop("visibility", UNSET)
        visibility: Union[Unset, visibility_m.Visibility]
        if isinstance(_visibility, Unset):
            visibility = UNSET
        else:
            visibility = visibility_m.Visibility.from_dict(_visibility)

        votes = d.pop("votes", UNSET)

        comments = []
        _comments = d.pop("comments", UNSET)
        for comments_item_data in _comments or []:
            comments_item = issue_comment_m.IssueComment.from_dict(comments_item_data)

            comments.append(comments_item)

        comments_count = d.pop("commentsCount", UNSET)

        tags = []
        _tags = d.pop("tags", UNSET)
        for tags_item_data in _tags or []:
            tags_item = issue_tag_m.IssueTag.from_dict(tags_item_data)

            tags.append(tags_item)

        links = []
        _links = d.pop("links", UNSET)
        for links_item_data in _links or []:
            links_item = issue_link_m.IssueLink.from_dict(links_item_data)

            links.append(links_item)

        _external_issue = d.pop("externalIssue", UNSET)
        external_issue: Union[Unset, external_issue_m.ExternalIssue]
        if isinstance(_external_issue, Unset):
            external_issue = UNSET
        else:
            external_issue = external_issue_m.ExternalIssue.from_dict(_external_issue)

        custom_fields = []
        _custom_fields = d.pop("customFields", UNSET)
        for custom_fields_item_data in _custom_fields or []:
            custom_fields_item = issue_custom_field_m.IssueCustomField.from_dict(custom_fields_item_data)

            custom_fields.append(custom_fields_item)

        _voters = d.pop("voters", UNSET)
        voters: Union[Unset, issue_voters_m.IssueVoters]
        if isinstance(_voters, Unset):
            voters = UNSET
        else:
            voters = issue_voters_m.IssueVoters.from_dict(_voters)

        _watchers = d.pop("watchers", UNSET)
        watchers: Union[Unset, issue_watchers_m.IssueWatchers]
        if isinstance(_watchers, Unset):
            watchers = UNSET
        else:
            watchers = issue_watchers_m.IssueWatchers.from_dict(_watchers)

        attachments = []
        _attachments = d.pop("attachments", UNSET)
        for attachments_item_data in _attachments or []:
            attachments_item = issue_attachment_m.IssueAttachment.from_dict(attachments_item_data)

            attachments.append(attachments_item)

        _subtasks = d.pop("subtasks", UNSET)
        subtasks: Union[Unset, issue_link_m.IssueLink]
        if isinstance(_subtasks, Unset):
            subtasks = UNSET
        else:
            subtasks = issue_link_m.IssueLink.from_dict(_subtasks)

        _parent = d.pop("parent", UNSET)
        parent: Union[Unset, issue_link_m.IssueLink]
        if isinstance(_parent, Unset):
            parent = UNSET
        else:
            parent = issue_link_m.IssueLink.from_dict(_parent)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        issue = cls(
            id_readable=id_readable,
            created=created,
            updated=updated,
            resolved=resolved,
            number_in_project=number_in_project,
            project=project,
            summary=summary,
            description=description,
            uses_markdown=uses_markdown,
            wikified_description=wikified_description,
            reporter=reporter,
            updater=updater,
            draft_owner=draft_owner,
            is_draft=is_draft,
            visibility=visibility,
            votes=votes,
            comments=comments,
            comments_count=comments_count,
            tags=tags,
            links=links,
            external_issue=external_issue,
            custom_fields=custom_fields,
            voters=voters,
            watchers=watchers,
            attachments=attachments,
            subtasks=subtasks,
            parent=parent,
            id=id,
            type=type,
        )

        return issue



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="DuplicateVote")


@attr.s(auto_attribs=True)
class DuplicateVote:
    """Represents a vote for duplicates of the issue."""

    issue: "Union[Unset, issue_m.Issue]" = UNSET
    user: "Union[Unset, user_m.User]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        issue: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.issue, Unset):
            issue = self.issue.to_dict()

        user: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.user, Unset):
            user = self.user.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if issue is not UNSET:
            field_dict["issue"] = issue
        if user is not UNSET:
            field_dict["user"] = user
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import issue as issue_m
            from ..models import user as user_m
        except ImportError:
            import sys

            issue_m = sys.modules[__package__ + "issue"]
            user_m = sys.modules[__package__ + "user"]

        d = src_dict.copy()

        _issue = d.pop("issue", UNSET)
        issue: Union[Unset, issue_m.Issue]
        if isinstance(_issue, Unset):
            issue = UNSET
        else:
            issue = issue_m.Issue.from_dict(_issue)

        _user = d.pop("user", UNSET)
        user: Union[Unset, user_m.User]
        if isinstance(_user, Unset):
            user = UNSET
        else:
            user = user_m.User.from_dict(_user)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        duplicate_vote = cls(
            issue=issue,
            user=user,
            id=id,
            type=type,
        )

        return duplicate_vote



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="IssueWorkItem")


@attr.s(auto_attribs=True)
class IssueWorkItem:
    """Represents a work item in an issue."""

    author: "Union[Unset, user_m.User]" = UNSET
    creator: "Union[Unset, user_m.User]" = UNSET
    text: "Union[Unset, str]" = UNSET
    text_preview: "Union[Unset, str]" = UNSET
    type: "Union[Unset, work_item_type_m.WorkItemType]" = UNSET
    created: "Union[Unset, int]" = UNSET
    updated: "Union[Unset, int]" = UNSET
    duration: "Union[Unset, duration_value_m.DurationValue]" = UNSET
    date: "Union[Unset, int]" = UNSET
    issue: "Union[Unset, issue_m.Issue]" = UNSET
    uses_markdown: "Union[Unset, bool]" = UNSET
    id: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        author: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.author, Unset):
            author = self.author.to_dict()

        creator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.creator, Unset):
            creator = self.creator.to_dict()

        text = self.text
        text_preview = self.text_preview
        type: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.to_dict()

        created = self.created
        updated = self.updated
        duration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.duration, Unset):
            duration = self.duration.to_dict()

        date = self.date
        issue: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.issue, Unset):
            issue = self.issue.to_dict()

        uses_markdown = self.uses_markdown
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if author is not UNSET:
            field_dict["author"] = author
        if creator is not UNSET:
            field_dict["creator"] = creator
        if text is not UNSET:
            field_dict["text"] = text
        if text_preview is not UNSET:
            field_dict["textPreview"] = text_preview
        if type is not UNSET:
            field_dict["type"] = type
        if created is not UNSET:
            field_dict["created"] = created
        if updated is not UNSET:
            field_dict["updated"] = updated
        if duration is not UNSET:
            field_dict["duration"] = duration
        if date is not UNSET:
            field_dict["date"] = date
        if issue is not UNSET:
            field_dict["issue"] = issue
        if uses_markdown is not UNSET:
            field_dict["usesMarkdown"] = uses_markdown
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import duration_value as duration_value_m
            from ..models import issue as issue_m
            from ..models import user as user_m
            from ..models import work_item_type as work_item_type_m
        except ImportError:
            import sys

            issue_m = sys.modules[__package__ + "issue"]
            duration_value_m = sys.modules[__package__ + "duration_value"]
            user_m = sys.modules[__package__ + "user"]
            work_item_type_m = sys.modules[__package__ + "work_item_type"]

        d = src_dict.copy()

        _author = d.pop("author", UNSET)
        author: Union[Unset, user_m.User]
        if isinstance(_author, Unset):
            author = UNSET
        else:
            author = user_m.User.from_dict(_author)

        _creator = d.pop("creator", UNSET)
        creator: Union[Unset, user_m.User]
        if isinstance(_creator, Unset):
            creator = UNSET
        else:
            creator = user_m.User.from_dict(_creator)

        text = d.pop("text", UNSET)

        text_preview = d.pop("textPreview", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, work_item_type_m.WorkItemType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = work_item_type_m.WorkItemType.from_dict(_type)

        created = d.pop("created", UNSET)

        updated = d.pop("updated", UNSET)

        _duration = d.pop("duration", UNSET)
        duration: Union[Unset, duration_value_m.DurationValue]
        if isinstance(_duration, Unset):
            duration = UNSET
        else:
            duration = duration_value_m.DurationValue.from_dict(_duration)

        date = d.pop("date", UNSET)

        _issue = d.pop("issue", UNSET)
        issue: Union[Unset, issue_m.Issue]
        if isinstance(_issue, Unset):
            issue = UNSET
        else:
            issue = issue_m.Issue.from_dict(_issue)

        uses_markdown = d.pop("usesMarkdown", UNSET)

        id = d.pop("id", UNSET)

        issue_work_item = cls(
            author=author,
            creator=creator,
            text=text,
            text_preview=text_preview,
            type=type,
            created=created,
            updated=updated,
            duration=duration,
            date=date,
            issue=issue,
            uses_markdown=uses_markdown,
            id=id,
        )

        return issue_work_item



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Agile")


@attr.s(auto_attribs=True)
class Agile:
    """Represents an agile board configuration."""

    name: "Union[Unset, str]" = UNSET
    owner: "Union[Unset, user_m.User]" = UNSET
    visible_for: "Union[Unset, user_group_m.UserGroup]" = UNSET
    visible_for_project_based: "Union[Unset, bool]" = UNSET
    updateable_by: "Union[Unset, user_group_m.UserGroup]" = UNSET
    updateable_by_project_based: "Union[Unset, bool]" = UNSET
    orphans_at_the_top: "Union[Unset, bool]" = UNSET
    hide_orphans_swimlane: "Union[Unset, bool]" = UNSET
    estimation_field: "Union[Unset, custom_field_m.CustomField]" = UNSET
    original_estimation_field: "Union[Unset, custom_field_m.CustomField]" = UNSET
    projects: "Union[Unset, List[project_m.Project]]" = UNSET
    sprints: "Union[Unset, List[sprint_m.Sprint]]" = UNSET
    current_sprint: "Union[Unset, sprint_m.Sprint]" = UNSET
    column_settings: "Union[Unset, column_settings_m.ColumnSettings]" = UNSET
    swimlane_settings: "Union[Unset, swimlane_settings_m.SwimlaneSettings]" = UNSET
    sprints_settings: "Union[Unset, sprints_settings_m.SprintsSettings]" = UNSET
    color_coding: "Union[Unset, color_coding_m.ColorCoding]" = UNSET
    status: "Union[Unset, agile_status_m.AgileStatus]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        owner: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner, Unset):
            owner = self.owner.to_dict()

        visible_for: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.visible_for, Unset):
            visible_for = self.visible_for.to_dict()

        visible_for_project_based = self.visible_for_project_based
        updateable_by: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.updateable_by, Unset):
            updateable_by = self.updateable_by.to_dict()

        updateable_by_project_based = self.updateable_by_project_based
        orphans_at_the_top = self.orphans_at_the_top
        hide_orphans_swimlane = self.hide_orphans_swimlane
        estimation_field: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.estimation_field, Unset):
            estimation_field = self.estimation_field.to_dict()

        original_estimation_field: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.original_estimation_field, Unset):
            original_estimation_field = self.original_estimation_field.to_dict()

        projects: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.projects, Unset):
            projects = []
            for projects_item_data in self.projects:
                projects_item = projects_item_data.to_dict()

                projects.append(projects_item)

        sprints: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.sprints, Unset):
            sprints = []
            for sprints_item_data in self.sprints:
                sprints_item = sprints_item_data.to_dict()

                sprints.append(sprints_item)

        current_sprint: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.current_sprint, Unset):
            current_sprint = self.current_sprint.to_dict()

        column_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.column_settings, Unset):
            column_settings = self.column_settings.to_dict()

        swimlane_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.swimlane_settings, Unset):
            swimlane_settings = self.swimlane_settings.to_dict()

        sprints_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.sprints_settings, Unset):
            sprints_settings = self.sprints_settings.to_dict()

        color_coding: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.color_coding, Unset):
            color_coding = self.color_coding.to_dict()

        status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if owner is not UNSET:
            field_dict["owner"] = owner
        if visible_for is not UNSET:
            field_dict["visibleFor"] = visible_for
        if visible_for_project_based is not UNSET:
            field_dict["visibleForProjectBased"] = visible_for_project_based
        if updateable_by is not UNSET:
            field_dict["updateableBy"] = updateable_by
        if updateable_by_project_based is not UNSET:
            field_dict["updateableByProjectBased"] = updateable_by_project_based
        if orphans_at_the_top is not UNSET:
            field_dict["orphansAtTheTop"] = orphans_at_the_top
        if hide_orphans_swimlane is not UNSET:
            field_dict["hideOrphansSwimlane"] = hide_orphans_swimlane
        if estimation_field is not UNSET:
            field_dict["estimationField"] = estimation_field
        if original_estimation_field is not UNSET:
            field_dict["originalEstimationField"] = original_estimation_field
        if projects is not UNSET:
            field_dict["projects"] = projects
        if sprints is not UNSET:
            field_dict["sprints"] = sprints
        if current_sprint is not UNSET:
            field_dict["currentSprint"] = current_sprint
        if column_settings is not UNSET:
            field_dict["columnSettings"] = column_settings
        if swimlane_settings is not UNSET:
            field_dict["swimlaneSettings"] = swimlane_settings
        if sprints_settings is not UNSET:
            field_dict["sprintsSettings"] = sprints_settings
        if color_coding is not UNSET:
            field_dict["colorCoding"] = color_coding
        if status is not UNSET:
            field_dict["status"] = status
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import agile_status as agile_status_m
            from ..models import color_coding as color_coding_m
            from ..models import column_settings as column_settings_m
            from ..models import custom_field as custom_field_m
            from ..models import project as project_m
            from ..models import sprint as sprint_m
            from ..models import sprints_settings as sprints_settings_m
            from ..models import swimlane_settings as swimlane_settings_m
            from ..models import user as user_m
            from ..models import user_group as user_group_m
        except ImportError:
            import sys

            project_m = sys.modules[__package__ + "project"]
            sprint_m = sys.modules[__package__ + "sprint"]
            user_m = sys.modules[__package__ + "user"]
            sprints_settings_m = sys.modules[__package__ + "sprints_settings"]
            user_group_m = sys.modules[__package__ + "user_group"]
            color_coding_m = sys.modules[__package__ + "color_coding"]
            agile_status_m = sys.modules[__package__ + "agile_status"]
            swimlane_settings_m = sys.modules[__package__ + "swimlane_settings"]
            column_settings_m = sys.modules[__package__ + "column_settings"]
            custom_field_m = sys.modules[__package__ + "custom_field"]

        d = src_dict.copy()

        name = d.pop("name", UNSET)

        _owner = d.pop("owner", UNSET)
        owner: Union[Unset, user_m.User]
        if isinstance(_owner, Unset):
            owner = UNSET
        else:
            owner = user_m.User.from_dict(_owner)

        _visible_for = d.pop("visibleFor", UNSET)
        visible_for: Union[Unset, user_group_m.UserGroup]
        if isinstance(_visible_for, Unset):
            visible_for = UNSET
        else:
            visible_for = user_group_m.UserGroup.from_dict(_visible_for)

        visible_for_project_based = d.pop("visibleForProjectBased", UNSET)

        _updateable_by = d.pop("updateableBy", UNSET)
        updateable_by: Union[Unset, user_group_m.UserGroup]
        if isinstance(_updateable_by, Unset):
            updateable_by = UNSET
        else:
            updateable_by = user_group_m.UserGroup.from_dict(_updateable_by)

        updateable_by_project_based = d.pop("updateableByProjectBased", UNSET)

        orphans_at_the_top = d.pop("orphansAtTheTop", UNSET)

        hide_orphans_swimlane = d.pop("hideOrphansSwimlane", UNSET)

        _estimation_field = d.pop("estimationField", UNSET)
        estimation_field: Union[Unset, custom_field_m.CustomField]
        if isinstance(_estimation_field, Unset):
            estimation_field = UNSET
        else:
            estimation_field = custom_field_m.CustomField.from_dict(_estimation_field)

        _original_estimation_field = d.pop("originalEstimationField", UNSET)
        original_estimation_field: Union[Unset, custom_field_m.CustomField]
        if isinstance(_original_estimation_field, Unset):
            original_estimation_field = UNSET
        else:
            original_estimation_field = custom_field_m.CustomField.from_dict(_original_estimation_field)

        projects = []
        _projects = d.pop("projects", UNSET)
        for projects_item_data in _projects or []:
            projects_item = project_m.Project.from_dict(projects_item_data)

            projects.append(projects_item)

        sprints = []
        _sprints = d.pop("sprints", UNSET)
        for sprints_item_data in _sprints or []:
            sprints_item = sprint_m.Sprint.from_dict(sprints_item_data)

            sprints.append(sprints_item)

        _current_sprint = d.pop("currentSprint", UNSET)
        current_sprint: Union[Unset, sprint_m.Sprint]
        if isinstance(_current_sprint, Unset):
            current_sprint = UNSET
        else:
            current_sprint = sprint_m.Sprint.from_dict(_current_sprint)

        _column_settings = d.pop("columnSettings", UNSET)
        column_settings: Union[Unset, column_settings_m.ColumnSettings]
        if isinstance(_column_settings, Unset):
            column_settings = UNSET
        else:
            column_settings = column_settings_m.ColumnSettings.from_dict(_column_settings)

        _swimlane_settings = d.pop("swimlaneSettings", UNSET)
        swimlane_settings: Union[Unset, swimlane_settings_m.SwimlaneSettings]
        if isinstance(_swimlane_settings, Unset):
            swimlane_settings = UNSET
        else:
            swimlane_settings = swimlane_settings_m.SwimlaneSettings.from_dict(_swimlane_settings)

        _sprints_settings = d.pop("sprintsSettings", UNSET)
        sprints_settings: Union[Unset, sprints_settings_m.SprintsSettings]
        if isinstance(_sprints_settings, Unset):
            sprints_settings = UNSET
        else:
            sprints_settings = sprints_settings_m.SprintsSettings.from_dict(_sprints_settings)

        _color_coding = d.pop("colorCoding", UNSET)
        color_coding: Union[Unset, color_coding_m.ColorCoding]
        if isinstance(_color_coding, Unset):
            color_coding = UNSET
        else:
            color_coding = color_coding_m.ColorCoding.from_dict(_color_coding)

        _status = d.pop("status", UNSET)
        status: Union[Unset, agile_status_m.AgileStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = agile_status_m.AgileStatus.from_dict(_status)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        agile = cls(
            name=name,
            owner=owner,
            visible_for=visible_for,
            visible_for_project_based=visible_for_project_based,
            updateable_by=updateable_by,
            updateable_by_project_based=updateable_by_project_based,
            orphans_at_the_top=orphans_at_the_top,
            hide_orphans_swimlane=hide_orphans_swimlane,
            estimation_field=estimation_field,
            original_estimation_field=original_estimation_field,
            projects=projects,
            sprints=sprints,
            current_sprint=current_sprint,
            column_settings=column_settings,
            swimlane_settings=swimlane_settings,
            sprints_settings=sprints_settings,
            color_coding=color_coding,
            status=status,
            id=id,
            type=type,
        )

        return agile



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GlobalSettings")


@attr.s(auto_attribs=True)
class GlobalSettings:
    """Represents application-wide settings."""

    system_settings: "Union[Unset, system_settings_m.SystemSettings]" = UNSET
    notification_settings: "Union[Unset, notification_settings_m.NotificationSettings]" = UNSET
    rest_settings: "Union[Unset, rest_cors_settings_m.RestCorsSettings]" = UNSET
    appearance_settings: "Union[Unset, appearance_settings_m.AppearanceSettings]" = UNSET
    locale_settings: "Union[Unset, locale_settings_m.LocaleSettings]" = UNSET
    license_: "Union[Unset, license__m.License]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        system_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.system_settings, Unset):
            system_settings = self.system_settings.to_dict()

        notification_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.notification_settings, Unset):
            notification_settings = self.notification_settings.to_dict()

        rest_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.rest_settings, Unset):
            rest_settings = self.rest_settings.to_dict()

        appearance_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.appearance_settings, Unset):
            appearance_settings = self.appearance_settings.to_dict()

        locale_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.locale_settings, Unset):
            locale_settings = self.locale_settings.to_dict()

        license_: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.license_, Unset):
            license_ = self.license_.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if system_settings is not UNSET:
            field_dict["systemSettings"] = system_settings
        if notification_settings is not UNSET:
            field_dict["notificationSettings"] = notification_settings
        if rest_settings is not UNSET:
            field_dict["restSettings"] = rest_settings
        if appearance_settings is not UNSET:
            field_dict["appearanceSettings"] = appearance_settings
        if locale_settings is not UNSET:
            field_dict["localeSettings"] = locale_settings
        if license_ is not UNSET:
            field_dict["license"] = license_
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import appearance_settings as appearance_settings_m
            from ..models import license_ as license__m
            from ..models import locale_settings as locale_settings_m
            from ..models import notification_settings as notification_settings_m
            from ..models import rest_cors_settings as rest_cors_settings_m
            from ..models import system_settings as system_settings_m
        except ImportError:
            import sys

            notification_settings_m = sys.modules[__package__ + "notification_settings"]
            system_settings_m = sys.modules[__package__ + "system_settings"]
            rest_cors_settings_m = sys.modules[__package__ + "rest_cors_settings"]
            locale_settings_m = sys.modules[__package__ + "locale_settings"]
            license__m = sys.modules[__package__ + "license_"]
            appearance_settings_m = sys.modules[__package__ + "appearance_settings"]

        d = src_dict.copy()

        _system_settings = d.pop("systemSettings", UNSET)
        system_settings: Union[Unset, system_settings_m.SystemSettings]
        if isinstance(_system_settings, Unset):
            system_settings = UNSET
        else:
            system_settings = system_settings_m.SystemSettings.from_dict(_system_settings)

        _notification_settings = d.pop("notificationSettings", UNSET)
        notification_settings: Union[Unset, notification_settings_m.NotificationSettings]
        if isinstance(_notification_settings, Unset):
            notification_settings = UNSET
        else:
            notification_settings = notification_settings_m.NotificationSettings.from_dict(_notification_settings)

        _rest_settings = d.pop("restSettings", UNSET)
        rest_settings: Union[Unset, rest_cors_settings_m.RestCorsSettings]
        if isinstance(_rest_settings, Unset):
            rest_settings = UNSET
        else:
            rest_settings = rest_cors_settings_m.RestCorsSettings.from_dict(_rest_settings)

        _appearance_settings = d.pop("appearanceSettings", UNSET)
        appearance_settings: Union[Unset, appearance_settings_m.AppearanceSettings]
        if isinstance(_appearance_settings, Unset):
            appearance_settings = UNSET
        else:
            appearance_settings = appearance_settings_m.AppearanceSettings.from_dict(_appearance_settings)

        _locale_settings = d.pop("localeSettings", UNSET)
        locale_settings: Union[Unset, locale_settings_m.LocaleSettings]
        if isinstance(_locale_settings, Unset):
            locale_settings = UNSET
        else:
            locale_settings = locale_settings_m.LocaleSettings.from_dict(_locale_settings)

        _license_ = d.pop("license", UNSET)
        license_: Union[Unset, license__m.License]
        if isinstance(_license_, Unset):
            license_ = UNSET
        else:
            license_ = license__m.License.from_dict(_license_)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        global_settings = cls(
            system_settings=system_settings,
            notification_settings=notification_settings,
            rest_settings=rest_settings,
            appearance_settings=appearance_settings,
            locale_settings=locale_settings,
            license_=license_,
            id=id,
            type=type,
        )

        return global_settings



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="TimeTrackingUserProfile")


@attr.s(auto_attribs=True)
class TimeTrackingUserProfile:
    """Represents time tracking settings in the user's profile."""

    period_format: "Union[Unset, period_field_format_m.PeriodFieldFormat]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        period_format: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.period_format, Unset):
            period_format = self.period_format.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if period_format is not UNSET:
            field_dict["periodFormat"] = period_format
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import period_field_format as period_field_format_m
        except ImportError:
            import sys

            period_field_format_m = sys.modules[__package__ + "period_field_format"]

        d = src_dict.copy()

        _period_format = d.pop("periodFormat", UNSET)
        period_format: Union[Unset, period_field_format_m.PeriodFieldFormat]
        if isinstance(_period_format, Unset):
            period_format = UNSET
        else:
            period_format = period_field_format_m.PeriodFieldFormat.from_dict(_period_format)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        time_tracking_user_profile = cls(
            period_format=period_format,
            id=id,
            type=type,
        )

        return time_tracking_user_profile



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="VcsChangeActivityItem")


try:
    from ..models import created_deleted_activity_item
except ImportError:
    import sys

    created_deleted_activity_item = sys.modules[__package__ + "created_deleted_activity_item"]


@attr.s(auto_attribs=True)
class VcsChangeActivityItem(created_deleted_activity_item.CreatedDeletedActivityItem):
    """Represents an update in the list of VCSChanges of an issue."""

    removed: "Union[Unset, vcs_change_activity_item_removed_m.VcsChangeActivityItemRemoved]" = UNSET
    added: "Union[Unset, vcs_change_activity_item_added_m.VcsChangeActivityItemAdded]" = UNSET
    author: "Union[Unset, user_m.User]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        removed: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.removed, Unset):
            removed = self.removed.to_dict()

        added: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.added, Unset):
            added = self.added.to_dict()

        author: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.author, Unset):
            author = self.author.to_dict()

        field_dict: Dict[str, Any] = {}
        _CreatedDeletedActivityItem_dict = super().to_dict()
        field_dict.update(_CreatedDeletedActivityItem_dict)
        field_dict.update({})
        if removed is not UNSET:
            field_dict["removed"] = removed
        if added is not UNSET:
            field_dict["added"] = added
        if author is not UNSET:
            field_dict["author"] = author

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import user as user_m
            from ..models import vcs_change_activity_item_added as vcs_change_activity_item_added_m
            from ..models import vcs_change_activity_item_removed as vcs_change_activity_item_removed_m
        except ImportError:
            import sys

            vcs_change_activity_item_removed_m = sys.modules[__package__ + "vcs_change_activity_item_removed"]
            user_m = sys.modules[__package__ + "user"]
            vcs_change_activity_item_added_m = sys.modules[__package__ + "vcs_change_activity_item_added"]

        d = src_dict.copy()

        _CreatedDeletedActivityItem_kwargs = super().from_dict(src_dict=d).to_dict()
        _CreatedDeletedActivityItem_kwargs.pop("$type")

        _removed = d.pop("removed", UNSET)
        removed: Union[Unset, vcs_change_activity_item_removed_m.VcsChangeActivityItemRemoved]
        if isinstance(_removed, Unset):
            removed = UNSET
        else:
            removed = vcs_change_activity_item_removed_m.VcsChangeActivityItemRemoved.from_dict(_removed)

        _added = d.pop("added", UNSET)
        added: Union[Unset, vcs_change_activity_item_added_m.VcsChangeActivityItemAdded]
        if isinstance(_added, Unset):
            added = UNSET
        else:
            added = vcs_change_activity_item_added_m.VcsChangeActivityItemAdded.from_dict(_added)

        _author = d.pop("author", UNSET)
        author: Union[Unset, user_m.User]
        if isinstance(_author, Unset):
            author = UNSET
        else:
            author = user_m.User.from_dict(_author)

        vcs_change_activity_item = cls(
            removed=removed,
            added=added,
            author=author,
            **_CreatedDeletedActivityItem_kwargs,
        )

        return vcs_change_activity_item



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="AgileColumn")


@attr.s(auto_attribs=True)
class AgileColumn:
    """Represents settings for a single board column"""

    presentation: "Union[Unset, str]" = UNSET
    is_resolved: "Union[Unset, bool]" = UNSET
    ordinal: "Union[Unset, int]" = UNSET
    parent: "Union[Unset, column_settings_m.ColumnSettings]" = UNSET
    wip_limit: "Union[Unset, wip_limit_m.WIPLimit]" = UNSET
    field_values: "Union[Unset, List[agile_column_field_value_m.AgileColumnFieldValue]]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        presentation = self.presentation
        is_resolved = self.is_resolved
        ordinal = self.ordinal
        parent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parent, Unset):
            parent = self.parent.to_dict()

        wip_limit: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.wip_limit, Unset):
            wip_limit = self.wip_limit.to_dict()

        field_values: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.field_values, Unset):
            field_values = []
            for field_values_item_data in self.field_values:
                field_values_item = field_values_item_data.to_dict()

                field_values.append(field_values_item)

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if presentation is not UNSET:
            field_dict["presentation"] = presentation
        if is_resolved is not UNSET:
            field_dict["isResolved"] = is_resolved
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if parent is not UNSET:
            field_dict["parent"] = parent
        if wip_limit is not UNSET:
            field_dict["wipLimit"] = wip_limit
        if field_values is not UNSET:
            field_dict["fieldValues"] = field_values
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import agile_column_field_value as agile_column_field_value_m
            from ..models import column_settings as column_settings_m
            from ..models import wip_limit as wip_limit_m
        except ImportError:
            import sys

            wip_limit_m = sys.modules[__package__ + "wip_limit"]
            agile_column_field_value_m = sys.modules[__package__ + "agile_column_field_value"]
            column_settings_m = sys.modules[__package__ + "column_settings"]

        d = src_dict.copy()

        presentation = d.pop("presentation", UNSET)

        is_resolved = d.pop("isResolved", UNSET)

        ordinal = d.pop("ordinal", UNSET)

        _parent = d.pop("parent", UNSET)
        parent: Union[Unset, column_settings_m.ColumnSettings]
        if isinstance(_parent, Unset):
            parent = UNSET
        else:
            parent = column_settings_m.ColumnSettings.from_dict(_parent)

        _wip_limit = d.pop("wipLimit", UNSET)
        wip_limit: Union[Unset, wip_limit_m.WIPLimit]
        if isinstance(_wip_limit, Unset):
            wip_limit = UNSET
        else:
            wip_limit = wip_limit_m.WIPLimit.from_dict(_wip_limit)

        field_values = []
        _field_values = d.pop("fieldValues", UNSET)
        for field_values_item_data in _field_values or []:
            field_values_item = agile_column_field_value_m.AgileColumnFieldValue.from_dict(field_values_item_data)

            field_values.append(field_values_item)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        agile_column = cls(
            presentation=presentation,
            is_resolved=is_resolved,
            ordinal=ordinal,
            parent=parent,
            wip_limit=wip_limit,
            field_values=field_values,
            id=id,
            type=type,
        )

        return agile_column



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="StateBundleElement")


try:
    from ..models import localizable_bundle_element
except ImportError:
    import sys

    localizable_bundle_element = sys.modules[__package__ + "localizable_bundle_element"]


@attr.s(auto_attribs=True)
class StateBundleElement(localizable_bundle_element.LocalizableBundleElement):
    """Represents the state of an issue in YouTrack."""

    is_resolved: "Union[Unset, bool]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        is_resolved = self.is_resolved

        field_dict: Dict[str, Any] = {}
        _LocalizableBundleElement_dict = super().to_dict()
        field_dict.update(_LocalizableBundleElement_dict)
        field_dict.update({})
        if is_resolved is not UNSET:
            field_dict["isResolved"] = is_resolved

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        _LocalizableBundleElement_kwargs = super().from_dict(src_dict=d).to_dict()
        _LocalizableBundleElement_kwargs.pop("$type")

        is_resolved = d.pop("isResolved", UNSET)

        state_bundle_element = cls(
            is_resolved=is_resolved,
            **_LocalizableBundleElement_kwargs,
        )

        return state_bundle_element



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="IssueComment")


@attr.s(auto_attribs=True)
class IssueComment:
    """Represents an existing issue comment."""

    text: "Union[Unset, str]" = UNSET
    uses_markdown: "Union[Unset, bool]" = UNSET
    text_preview: "Union[Unset, str]" = UNSET
    created: "Union[Unset, int]" = UNSET
    updated: "Union[Unset, int]" = UNSET
    author: "Union[Unset, user_m.User]" = UNSET
    issue: "Union[Unset, issue_m.Issue]" = UNSET
    attachments: "Union[Unset, List[issue_attachment_m.IssueAttachment]]" = UNSET
    visibility: "Union[Unset, visibility_m.Visibility]" = UNSET
    deleted: "Union[Unset, bool]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        text = self.text
        uses_markdown = self.uses_markdown
        text_preview = self.text_preview
        created = self.created
        updated = self.updated
        author: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.author, Unset):
            author = self.author.to_dict()

        issue: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.issue, Unset):
            issue = self.issue.to_dict()

        attachments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.attachments, Unset):
            attachments = []
            for attachments_item_data in self.attachments:
                attachments_item = attachments_item_data.to_dict()

                attachments.append(attachments_item)

        visibility: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.visibility, Unset):
            visibility = self.visibility.to_dict()

        deleted = self.deleted
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if text is not UNSET:
            field_dict["text"] = text
        if uses_markdown is not UNSET:
            field_dict["usesMarkdown"] = uses_markdown
        if text_preview is not UNSET:
            field_dict["textPreview"] = text_preview
        if created is not UNSET:
            field_dict["created"] = created
        if updated is not UNSET:
            field_dict["updated"] = updated
        if author is not UNSET:
            field_dict["author"] = author
        if issue is not UNSET:
            field_dict["issue"] = issue
        if attachments is not UNSET:
            field_dict["attachments"] = attachments
        if visibility is not UNSET:
            field_dict["visibility"] = visibility
        if deleted is not UNSET:
            field_dict["deleted"] = deleted
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import issue as issue_m
            from ..models import issue_attachment as issue_attachment_m
            from ..models import user as user_m
            from ..models import visibility as visibility_m
        except ImportError:
            import sys

            issue_m = sys.modules[__package__ + "issue"]
            visibility_m = sys.modules[__package__ + "visibility"]
            user_m = sys.modules[__package__ + "user"]
            issue_attachment_m = sys.modules[__package__ + "issue_attachment"]

        d = src_dict.copy()

        text = d.pop("text", UNSET)

        uses_markdown = d.pop("usesMarkdown", UNSET)

        text_preview = d.pop("textPreview", UNSET)

        created = d.pop("created", UNSET)

        updated = d.pop("updated", UNSET)

        _author = d.pop("author", UNSET)
        author: Union[Unset, user_m.User]
        if isinstance(_author, Unset):
            author = UNSET
        else:
            author = user_m.User.from_dict(_author)

        _issue = d.pop("issue", UNSET)
        issue: Union[Unset, issue_m.Issue]
        if isinstance(_issue, Unset):
            issue = UNSET
        else:
            issue = issue_m.Issue.from_dict(_issue)

        attachments = []
        _attachments = d.pop("attachments", UNSET)
        for attachments_item_data in _attachments or []:
            attachments_item = issue_attachment_m.IssueAttachment.from_dict(attachments_item_data)

            attachments.append(attachments_item)

        _visibility = d.pop("visibility", UNSET)
        visibility: Union[Unset, visibility_m.Visibility]
        if isinstance(_visibility, Unset):
            visibility = UNSET
        else:
            visibility = visibility_m.Visibility.from_dict(_visibility)

        deleted = d.pop("deleted", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        issue_comment = cls(
            text=text,
            uses_markdown=uses_markdown,
            text_preview=text_preview,
            created=created,
            updated=updated,
            author=author,
            issue=issue,
            attachments=attachments,
            visibility=visibility,
            deleted=deleted,
            id=id,
            type=type,
        )

        return issue_comment



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="BackupError")


@attr.s(auto_attribs=True)
class BackupError:
    """Represents an error that appeared during the backup."""

    date: "Union[Unset, int]" = UNSET
    error_message: "Union[Unset, str]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        date = self.date
        error_message = self.error_message
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if date is not UNSET:
            field_dict["date"] = date
        if error_message is not UNSET:
            field_dict["errorMessage"] = error_message
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        date = d.pop("date", UNSET)

        error_message = d.pop("errorMessage", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        backup_error = cls(
            date=date,
            error_message=error_message,
            id=id,
            type=type,
        )

        return backup_error



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ProjectBasedColorCoding")


try:
    from ..models import color_coding
except ImportError:
    import sys

    color_coding = sys.modules[__package__ + "color_coding"]


@attr.s(auto_attribs=True)
class ProjectBasedColorCoding(color_coding.ColorCoding):
    """Lets you set a color for a card based on its project."""

    project_colors: "Union[Unset, List[project_color_m.ProjectColor]]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        project_colors: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.project_colors, Unset):
            project_colors = []
            for project_colors_item_data in self.project_colors:
                project_colors_item = project_colors_item_data.to_dict()

                project_colors.append(project_colors_item)

        field_dict: Dict[str, Any] = {}
        _ColorCoding_dict = super().to_dict()
        field_dict.update(_ColorCoding_dict)
        field_dict.update({})
        if project_colors is not UNSET:
            field_dict["projectColors"] = project_colors

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import project_color as project_color_m
        except ImportError:
            import sys

            project_color_m = sys.modules[__package__ + "project_color"]

        d = src_dict.copy()

        _ColorCoding_kwargs = super().from_dict(src_dict=d).to_dict()
        _ColorCoding_kwargs.pop("$type")

        project_colors = []
        _project_colors = d.pop("projectColors", UNSET)
        for project_colors_item_data in _project_colors or []:
            project_colors_item = project_color_m.ProjectColor.from_dict(project_colors_item_data)

            project_colors.append(project_colors_item)

        project_based_color_coding = cls(
            project_colors=project_colors,
            **_ColorCoding_kwargs,
        )

        return project_based_color_coding



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="NotificationSettings")


@attr.s(auto_attribs=True)
class NotificationSettings:
    """Represents the Notifications settings of the YouTrack service."""

    email_settings: "Union[Unset, email_settings_m.EmailSettings]" = UNSET
    jabber_settings: "Union[Unset, jabber_settings_m.JabberSettings]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        email_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.email_settings, Unset):
            email_settings = self.email_settings.to_dict()

        jabber_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.jabber_settings, Unset):
            jabber_settings = self.jabber_settings.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if email_settings is not UNSET:
            field_dict["emailSettings"] = email_settings
        if jabber_settings is not UNSET:
            field_dict["jabberSettings"] = jabber_settings
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import email_settings as email_settings_m
            from ..models import jabber_settings as jabber_settings_m
        except ImportError:
            import sys

            email_settings_m = sys.modules[__package__ + "email_settings"]
            jabber_settings_m = sys.modules[__package__ + "jabber_settings"]

        d = src_dict.copy()

        _email_settings = d.pop("emailSettings", UNSET)
        email_settings: Union[Unset, email_settings_m.EmailSettings]
        if isinstance(_email_settings, Unset):
            email_settings = UNSET
        else:
            email_settings = email_settings_m.EmailSettings.from_dict(_email_settings)

        _jabber_settings = d.pop("jabberSettings", UNSET)
        jabber_settings: Union[Unset, jabber_settings_m.JabberSettings]
        if isinstance(_jabber_settings, Unset):
            jabber_settings = UNSET
        else:
            jabber_settings = jabber_settings_m.JabberSettings.from_dict(_jabber_settings)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        notification_settings = cls(
            email_settings=email_settings,
            jabber_settings=jabber_settings,
            id=id,
            type=type,
        )

        return notification_settings



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="AppearanceSettings")


@attr.s(auto_attribs=True)
class AppearanceSettings:
    """Represents the Visual settings of the YouTrack service."""

    time_zone: "Union[Unset, time_zone_descriptor_m.TimeZoneDescriptor]" = UNSET
    date_field_format: "Union[Unset, date_format_descriptor_m.DateFormatDescriptor]" = UNSET
    logo: "Union[Unset, logo_m.Logo]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        time_zone: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time_zone, Unset):
            time_zone = self.time_zone.to_dict()

        date_field_format: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.date_field_format, Unset):
            date_field_format = self.date_field_format.to_dict()

        logo: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.logo, Unset):
            logo = self.logo.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if time_zone is not UNSET:
            field_dict["timeZone"] = time_zone
        if date_field_format is not UNSET:
            field_dict["dateFieldFormat"] = date_field_format
        if logo is not UNSET:
            field_dict["logo"] = logo
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import date_format_descriptor as date_format_descriptor_m
            from ..models import logo as logo_m
            from ..models import time_zone_descriptor as time_zone_descriptor_m
        except ImportError:
            import sys

            time_zone_descriptor_m = sys.modules[__package__ + "time_zone_descriptor"]
            logo_m = sys.modules[__package__ + "logo"]
            date_format_descriptor_m = sys.modules[__package__ + "date_format_descriptor"]

        d = src_dict.copy()

        _time_zone = d.pop("timeZone", UNSET)
        time_zone: Union[Unset, time_zone_descriptor_m.TimeZoneDescriptor]
        if isinstance(_time_zone, Unset):
            time_zone = UNSET
        else:
            time_zone = time_zone_descriptor_m.TimeZoneDescriptor.from_dict(_time_zone)

        _date_field_format = d.pop("dateFieldFormat", UNSET)
        date_field_format: Union[Unset, date_format_descriptor_m.DateFormatDescriptor]
        if isinstance(_date_field_format, Unset):
            date_field_format = UNSET
        else:
            date_field_format = date_format_descriptor_m.DateFormatDescriptor.from_dict(_date_field_format)

        _logo = d.pop("logo", UNSET)
        logo: Union[Unset, logo_m.Logo]
        if isinstance(_logo, Unset):
            logo = UNSET
        else:
            logo = logo_m.Logo.from_dict(_logo)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        appearance_settings = cls(
            time_zone=time_zone,
            date_field_format=date_field_format,
            logo=logo,
            id=id,
            type=type,
        )

        return appearance_settings



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="TextFieldValue")


@attr.s(auto_attribs=True)
class TextFieldValue:
    """Represents a value of the text field. Returns both source and rendered text."""

    text: "Union[Unset, str]" = UNSET
    markdown_text: "Union[Unset, str]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        text = self.text
        markdown_text = self.markdown_text
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if text is not UNSET:
            field_dict["text"] = text
        if markdown_text is not UNSET:
            field_dict["markdownText"] = markdown_text
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        text = d.pop("text", UNSET)

        markdown_text = d.pop("markdownText", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        text_field_value = cls(
            text=text,
            markdown_text=markdown_text,
            id=id,
            type=type,
        )

        return text_field_value



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models import agile as agile_m
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    fields: "Union[Unset, None, str]" = "$type,columnSettings($type,columns($type,id),field($type,fieldType($type,id),id,localizedName,name),id),id,name,owner($type,id,login,name,ringId),projects($type,id,name,shortName),status($type,id,valid),swimlaneSettings($type,enabled,field($type,customField($type,fieldType($type,id),id,localizedName,name),id,name),id,values($type,id,name))",
    skip: "Union[Unset, None, int]" = UNSET,
    top: "Union[Unset, None, int]" = UNSET,
) -> Dict[str, Any]:
    url = "{}/agiles".format(client.youtrack_base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "fields": fields,
        "$skip": skip,
        "$top": top,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[agile_m.Agile]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = agile_m.Agile.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[List[agile_m.Agile]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    fields: "Union[Unset, None, str]" = "$type,columnSettings($type,columns($type,id),field($type,fieldType($type,id),id,localizedName,name),id),id,name,owner($type,id,login,name,ringId),projects($type,id,name,shortName),status($type,id,valid),swimlaneSettings($type,enabled,field($type,customField($type,fieldType($type,id),id,localizedName,name),id,name),id,values($type,id,name))",
    skip: "Union[Unset, None, int]" = UNSET,
    top: "Union[Unset, None, int]" = UNSET,
) -> Response[List[agile_m.Agile]]:
    kwargs = _get_kwargs(
        client=client,
        fields=fields,
        skip=skip,
        top=top,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    fields: "Union[Unset, None, str]" = "$type,columnSettings($type,columns($type,id),field($type,fieldType($type,id),id,localizedName,name),id),id,name,owner($type,id,login,name,ringId),projects($type,id,name,shortName),status($type,id,valid),swimlaneSettings($type,enabled,field($type,customField($type,fieldType($type,id),id,localizedName,name),id,name),id,values($type,id,name))",
    skip: "Union[Unset, None, int]" = UNSET,
    top: "Union[Unset, None, int]" = UNSET,
) -> Optional[List[agile_m.Agile]]:
    """ """

    return sync_detailed(
        client=client,
        fields=fields,
        skip=skip,
        top=top,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    fields: "Union[Unset, None, str]" = "$type,columnSettings($type,columns($type,id),field($type,fieldType($type,id),id,localizedName,name),id),id,name,owner($type,id,login,name,ringId),projects($type,id,name,shortName),status($type,id,valid),swimlaneSettings($type,enabled,field($type,customField($type,fieldType($type,id),id,localizedName,name),id,name),id,values($type,id,name))",
    skip: "Union[Unset, None, int]" = UNSET,
    top: "Union[Unset, None, int]" = UNSET,
) -> Response[List[agile_m.Agile]]:
    kwargs = _get_kwargs(
        client=client,
        fields=fields,
        skip=skip,
        top=top,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    fields: "Union[Unset, None, str]" = "$type,columnSettings($type,columns($type,id),field($type,fieldType($type,id),id,localizedName,name),id),id,name,owner($type,id,login,name,ringId),projects($type,id,name,shortName),status($type,id,valid),swimlaneSettings($type,enabled,field($type,customField($type,fieldType($type,id),id,localizedName,name),id,name),id,values($type,id,name))",
    skip: "Union[Unset, None, int]" = UNSET,
    top: "Union[Unset, None, int]" = UNSET,
) -> Optional[List[agile_m.Agile]]:
    """ """

    return (
        await asyncio_detailed(
            client=client,
            fields=fields,
            skip=skip,
            top=top,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="IssueLinkType")


@attr.s(auto_attribs=True)
class IssueLinkType:
    """Represents the settings of a link type in YouTrack."""

    name: "Union[Unset, str]" = UNSET
    localized_name: "Union[Unset, str]" = UNSET
    source_to_target: "Union[Unset, str]" = UNSET
    localized_source_to_target: "Union[Unset, str]" = UNSET
    target_to_source: "Union[Unset, str]" = UNSET
    localized_target_to_source: "Union[Unset, str]" = UNSET
    directed: "Union[Unset, bool]" = UNSET
    aggregation: "Union[Unset, bool]" = UNSET
    read_only: "Union[Unset, bool]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        localized_name = self.localized_name
        source_to_target = self.source_to_target
        localized_source_to_target = self.localized_source_to_target
        target_to_source = self.target_to_source
        localized_target_to_source = self.localized_target_to_source
        directed = self.directed
        aggregation = self.aggregation
        read_only = self.read_only
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if localized_name is not UNSET:
            field_dict["localizedName"] = localized_name
        if source_to_target is not UNSET:
            field_dict["sourceToTarget"] = source_to_target
        if localized_source_to_target is not UNSET:
            field_dict["localizedSourceToTarget"] = localized_source_to_target
        if target_to_source is not UNSET:
            field_dict["targetToSource"] = target_to_source
        if localized_target_to_source is not UNSET:
            field_dict["localizedTargetToSource"] = localized_target_to_source
        if directed is not UNSET:
            field_dict["directed"] = directed
        if aggregation is not UNSET:
            field_dict["aggregation"] = aggregation
        if read_only is not UNSET:
            field_dict["readOnly"] = read_only
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        name = d.pop("name", UNSET)

        localized_name = d.pop("localizedName", UNSET)

        source_to_target = d.pop("sourceToTarget", UNSET)

        localized_source_to_target = d.pop("localizedSourceToTarget", UNSET)

        target_to_source = d.pop("targetToSource", UNSET)

        localized_target_to_source = d.pop("localizedTargetToSource", UNSET)

        directed = d.pop("directed", UNSET)

        aggregation = d.pop("aggregation", UNSET)

        read_only = d.pop("readOnly", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        issue_link_type = cls(
            name=name,
            localized_name=localized_name,
            source_to_target=source_to_target,
            localized_source_to_target=localized_source_to_target,
            target_to_source=target_to_source,
            localized_target_to_source=localized_target_to_source,
            directed=directed,
            aggregation=aggregation,
            read_only=read_only,
            id=id,
            type=type,
        )

        return issue_link_type



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ProjectColor")


@attr.s(auto_attribs=True)
class ProjectColor:
    """Represents color setting for one project on the board."""

    project: "Union[Unset, project_m.Project]" = UNSET
    color: "Union[Unset, field_style_m.FieldStyle]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        project: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.project, Unset):
            project = self.project.to_dict()

        color: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.color, Unset):
            color = self.color.to_dict()

        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if project is not UNSET:
            field_dict["project"] = project
        if color is not UNSET:
            field_dict["color"] = color
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        try:
            from ..models import field_style as field_style_m
            from ..models import project as project_m
        except ImportError:
            import sys

            project_m = sys.modules[__package__ + "project"]
            field_style_m = sys.modules[__package__ + "field_style"]

        d = src_dict.copy()

        _project = d.pop("project", UNSET)
        project: Union[Unset, project_m.Project]
        if isinstance(_project, Unset):
            project = UNSET
        else:
            project = project_m.Project.from_dict(_project)

        _color = d.pop("color", UNSET)
        color: Union[Unset, field_style_m.FieldStyle]
        if isinstance(_color, Unset):
            color = UNSET
        else:
            color = field_style_m.FieldStyle.from_dict(_color)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        project_color = cls(
            project=project,
            color=color,
            id=id,
            type=type,
        )

        return project_color



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="AgileStatus")


@attr.s(auto_attribs=True)
class AgileStatus:
    """Shows if the board has any configuration problems."""

    valid: "Union[Unset, bool]" = UNSET
    has_jobs: "Union[Unset, bool]" = UNSET
    errors: "Union[Unset, str]" = UNSET
    warnings: "Union[Unset, str]" = UNSET
    id: "Union[Unset, str]" = UNSET
    type: "Union[Unset, str]" = UNSET

    def to_dict(self) -> Dict[str, Any]:
        valid = self.valid
        has_jobs = self.has_jobs
        errors = self.errors
        warnings = self.warnings
        id = self.id
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if valid is not UNSET:
            field_dict["valid"] = valid
        if has_jobs is not UNSET:
            field_dict["hasJobs"] = has_jobs
        if errors is not UNSET:
            field_dict["errors"] = errors
        if warnings is not UNSET:
            field_dict["warnings"] = warnings
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["$type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()

        valid = d.pop("valid", UNSET)

        has_jobs = d.pop("hasJobs", UNSET)

        errors = d.pop("errors", UNSET)

        warnings = d.pop("warnings", UNSET)

        id = d.pop("id", UNSET)

        type = d.pop("$type", UNSET)

        agile_status = cls(
            valid=valid,
            has_jobs=has_jobs,
            errors=errors,
            warnings=warnings,
            id=id,
            type=type,
        )

        return agile_status



from typing import Dict

import attr
from hub_rest_client import Client as HubClient


@attr.s(auto_attribs=True)
class Client(HubClient):
    """A class for keeping track of data related to the API"""

    youtrack_base_url: str = attr.ib("https://youtrack.jetbrains.com/api", kw_only=True)


@attr.s(auto_attribs=True)
class AuthenticatedClient(Client):
    """A Client which has been authenticated for use on secured endpoints"""

    token: str

    def get_headers(self) -> Dict[str, str]:
        """Get headers to be used in authenticated endpoints"""
        return {"Authorization": f"Bearer {self.token}", **self.headers}
